# Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import re
import traceback
import random
try:
    import syslog
    import fcntl
except ImportError:
    pass
import select
import time
import signal

import pikzie.core
import pikzie.pretty_print as pp

class Assertions(object):
    def fail(self, message):
        """
        Always fails with message.

          self.fail("must not happen!") # => fail
        """
        self._fail(message)

    def pend(self, message):
        """
        Pend the current running test.

          self.pend("module XXX isn't found") # => pend test
        """
        self._pend(message)

    def notify(self, message):
        """
        Notify a message for the current running test.

          if command_not_found:
              self.notify("skip due to command not found") # => notify a message
              return
        """
        self._notify(message)

    def omit(self, message):
        """
        Omit the current running test.

          if module_not_found:
              self.omit("omit due to a module isn't found") # => omit test
        """
        self._omit(message)

    def assert_none(self, expression, message=None):
        """
        Passes if expression is None.

          self.assert_none(None) # => pass
        """
        if expression is None:
            self._pass_assertion()
        else:
            system_message = "expected: <%r> is None" % expression
            self._fail(system_message, message)

    def assert_not_none(self, expression, message=None):
        """
        Passes if expression is not None.

          self.assert_not_none("not none") # => pass
        """
        if expression is not None:
            self._pass_assertion()
        else:
            self._fail("expected: not None", message)

    def assert_true(self, expression, message=None):
        """
        Passes if expression is true value.

          self.assert_true(True)     # => pass
          self.assert_true("string") # => pass
        """
        if expression:
            self._pass_assertion()
        else:
            system_message = "expected: <%r> is a true value" % expression
            self._fail(system_message, message)

    def assert_false(self, expression, message=None):
        """
        Passes if expression is false value.

          self.assert_false(False) # => pass
          self.assert_false("")    # => pass
        """
        if expression:
            system_message = "expected: <%r> is a false value" % expression
            self._fail(system_message, message)
        else:
            self._pass_assertion()

    def assert_equal(self, expected, actual, message=None):
        """
        Passes if expected == actual.

          self.assert_equal(5, 2 + 3) # => pass
        """
        if expected == actual:
            self._pass_assertion()
        else:
            formatted_expected = pp.format(expected)
            formatted_actual = pp.format(actual)
            self._fail("", message, formatted_expected, formatted_actual)

    def assert_not_equal(self, not_expected, actual, message=None):
        """
        Passes if not_expected != actual.

          self.assert_not_equal(-5, 2 + 3) # => pass
        """
        if not_expected != actual:
            self._pass_assertion()
        else:
            formatted_not_expected = pp.format(not_expected)
            formatted_actual = pp.format(actual)
            system_message = "not expected: <%s>\n     but was: <%s>" % \
                (formatted_not_expected, formatted_actual)
            if formatted_not_expected != formatted_actual:
                system_message = pp.append_diff(system_message,
                                                not_expected, actual)
            self._fail(system_message, message)

    def assert_in_delta(self, expected, actual, delta, message=None):
        """
        Passes if (expected - delta) <= actual <= (expected + delta).

          self.assert_in_delta(3, 3.01, 0.001) # => pass
        """
        lower = expected - delta
        upper = expected + delta
        if lower <= actual <= upper:
            self._pass_assertion()
        else:
            expected = pp.format(expected)
            actual = pp.format(actual)
            delta = pp.format(delta)
            range = pp.format([lower, upper])
            system_message = "expected: <%s+-%s %s>\n but was: <%s>" % \
                (expected, delta, range, actual)
            self._fail(system_message, message)

    def assert_match(self, pattern, target, message=None):
        """
        Passes if re.match(pattern, target) doesn't return None.

          self.assert_match("abc", "abcde") # => pass
          self.assert_match("abc", "deabc") # => fail
        """
        if re.match(pattern, target):
            self._pass_assertion()
        else:
            pattern_repr = pp.format_re_repr(pattern)
            pattern = pp.format_re(pattern)
            target = pp.format(target)
            format = \
                "expected: re.match(%s, %s) doesn't return None\n" \
                " pattern: <%s>\n" \
                "  target: <%s>"
            system_message = format % (pattern_repr, target, pattern, target)
            self._fail(system_message, message)

    def assert_not_match(self, pattern, target, message=None):
        """
        Passes if re.match(pattern, target) returns None.

          self.assert_not_match("abc", "deabc") # => pass
          self.assert_not_match("abc", "abcde") # => fail
        """
        if re.match(pattern, target) is None:
            self._pass_assertion()
        else:
            pattern_repr = pp.format_re_repr(pattern)
            pattern = pp.format_re(pattern)
            target = pp.format(target)
            format = \
                "expected: re.match(%s, %s) returns None\n" \
                " pattern: <%s>\n" \
                "  target: <%s>"
            system_message = format % (pattern_repr, target, pattern, target)
            self._fail(system_message, message)

    def assert_search(self, pattern, target, message=None):
        """
        Passes if re.search(pattern, target) doesn't return None.

        self.assert_search("abc", "deabc") # => pass
        self.assert_search("abc", "deABC") # => fail
        """
        if re.search(pattern, target):
            self._pass_assertion()
        else:
            pattern_repr = pp.format_re_repr(pattern)
            pattern = pp.format_re(pattern)
            target = pp.format(target)
            format = \
                "expected: re.search(%s, %s) doesn't return None\n" \
                " pattern: <%s>\n" \
                "  target: <%s>"
            system_message = format % (pattern_repr, target, pattern, target)
            self._fail(system_message, message)

    def assert_not_found(self, pattern, target, message=None):
        """
        Passes if re.search(pattern, target) returns None.

          self.assert_search("abc", "deABC") # => pass
          self.assert_search("abc", "deabc") # => fail
        """
        if re.search(pattern, target) is None:
            self._pass_assertion()
        else:
            pattern_repr = pp.format_re_repr(pattern)
            pattern = pp.format_re(pattern)
            target = pp.format(target)
            format = \
                "expected: re.search(%s, %s) returns None\n" \
                " pattern: <%s>\n" \
                "  target: <%s>"
            system_message = format % (pattern_repr, target, pattern, target)
            self._fail(system_message, message)

    def assert_hasattr(self, object, name, message=None):
        """
        Passes if hasattr(object, name) returns True.

          self.assert_hasattr("string", "strip")   # => pass
          self.assert_hasattr("string", "unknown") # => fail
        """
        if hasattr(object, name):
            self._pass_assertion()
        else:
            object = pp.format(object)
            name = pp.format(name)
            system_message = "expected: hasattr(%s, %s)" % (object, name)
            self._fail(system_message, message)

    def assert_callable(self, object, message=None):
        """
        Passes if callable(object) returns True.

          self.assert_callable(lambda: 1) # => pass
          self.assert_callable("string")  # => fail
        """
        if callable(object):
            self._pass_assertion()
        else:
            object = pp.format(object)
            system_message = "expected: callable(%s)" % (object)
            self._fail(system_message, message)

    def assert_raise_call(self, exception, callable_object, *args, **kw_args):
        """
        Passes if callable_object(*args, **kw_args) raises exception and
        returns raised exception value.

          self.assert_raise_call(NameError,
                                 lambda: unknown_variable) # => pass
                                                           # => returns NameError
                                                           #    value
          self.assert_raise_call(NameError, lambda: 1)     # => fail

        Exception instance can be also passed if it's comparable.

          class ComparableError(Exception):
              def __init__(self, message):
                  self.message = message

              def __repr__(self):
                  return "%s(%r,)" % (type(self).__name__, self.message)

              def __eq__(self, other):
                  return isinstance(other, self.__class__) and \
                          self.message == other.message

          def raise_error():
              raise ComparableError("value")
          self.assert_raise_call(ComparableError("value"),
                                 raise_error)              # => pass
                                                           # => returns
                                                           #    ComparableError
                                                           #    value
          self.assert_raise_call(ComparableError("key"),
                                 raise_error)              # => fail
        """
        self.assert_callable(callable_object)
        if isinstance(exception, Exception):
            exception_class = exception.__class__
        else:
            exception_class = exception
        try:
            callable_object(*args, **kw_args)
        except exception_class:
            actual = sys.exc_info()[1]
            if exception_class == exception:
                self._pass_assertion()
            else:
                self.assert_equal(exception, actual)
            return actual
        except:
            actual = sys.exc_info()
            actual_exception_class, actual_exception_value = actual[:2]
            if exception_class == exception:
                expected = "<%s>" % pp.format_exception_class(exception_class)
            else:
                expected = "<%s>(%s)" % \
                    (pp.format_exception_class(exception_class),
                     str(exception))
            actual = "<%s>(%s)" % \
                (pp.format_exception_class(actual_exception_class),
                 str(actual_exception_value))
            message = \
                "expected: %s is raised\n" \
                " but was: %s" % (expected, actual)
            self._fail(message)
        else:
            message = \
                "expected: <%s> is raised\n" \
                " but was: %s nothing raised" % \
                (pp.format_exception_class(exception_class),
                 pp.format_call(callable_object, args, kw_args))
            self._fail(message)

    def assert_call_raise(self, *args, **kw_args):
        """Deprecated. Use assert_raise_call()."""
        self.notify("assert_call_raise() is deprecated. "
                    "Use assert_raise_call() instead.")
        return self.assert_raise_call(*args, **kw_args)

    def assert_nothing_raised_call(self, callable_object, *args, **kw_args):
        """
        Passes if callable_object(*args, **kw_args) raises nothing exception
        and returns called result.

          self.assert_nothing_raised_call(lambda: 1)                # => pass
                                                                    # => returns 1
          self.assert_nothing_raised_call(lambda: unknown_variable) # => fail
        """
        self.assert_callable(callable_object)
        try:
            result = callable_object(*args, **kw_args)
        except:
            actual = sys.exc_info()[:2]
            actual_exception_class, actual_exception_value = actual
            message = \
                "expected: %s nothing raised\n" \
                " but was: <%s>(%s) is raised" % \
                (pp.format_call(callable_object, args, kw_args),
                 pp.format_exception_class(actual_exception_class),
                 str(actual_exception_value))
            self._fail(message)
        self._pass_assertion()
        return result

    def assert_call_nothing_raised(self, *args, **kw_args):
        """Deprecated. Use assert_nothing_raised_call()."""
        self.notify("assert_call_nothing_raised() is deprecated. "
                    "Use assert_nothing_raised_call() instead.")
        return self.assert_nothing_raised_call(*args, **kw_args)

    def assert_run_command(self, command, **kw_args):
        """
        Passes if command is successfully ran and returns subprocess.Popen.

          process = self.assert_run_command(["echo", "123"])    # => pass
          self.assert_equal("123\\n", process.stdout.read())    # => pass
          self.assert_run_command("false")                      # => fail
          self.assert_run_command("unknown-command")            # => fail
        """
        import subprocess
        popen_kw_args = {
            "stdin": subprocess.PIPE,
            "stdout": subprocess.PIPE,
            "stderr": subprocess.PIPE,
        }
        popen_kw_args.update(kw_args)
        try:
            process = subprocess.Popen(command, **popen_kw_args)
        except OSError:
            exception_class, exception_value = sys.exc_info()[:2]
            message = "expected: <%s> is successfully ran\n" \
                " but was: <%s>(%s) is raised and failed to ran" % \
                (pp.format(command),
                 pp.format_exception_class(exception_class),
                 str(exception_value))
            self._fail(message)
        return_code = process.wait()
        if return_code != 0:
            message = "expected: <%s> is successfully finished\n" \
                " but was: <%d> is returned as exit code" % \
                (pp.format(command), return_code)
            self._fail(message)
        self._pass_assertion()
        return process

    def assert_search_syslog_call(self, pattern,
                                  callable_object, *args, **kw_args):
        """
        Passes if re.search(pattern, SYSLOG_CONTENT) doesn't return None
        after callable_object(*args, **kw_args).

          self.assert_search_syslog_call("X", syslog.syslog, "XYZ") # => pass
          self.assert_search_syslog_call("X", syslog.syslog, "ABC") # => fail
        """
        if not hasattr(sys.modules[__name__], "syslog"):
            self.omit("syslog isn't supported on this environment")

        self.assert_callable(callable_object)

        mark = 'Pikzie: %.20f' % random.random()
        syslog.syslog(mark)

        log_file = "/var/log/messages"
        command = ["tail", "-F", log_file]
        try:
            from subprocess import Popen, PIPE
            messages = Popen(command, stdout=PIPE, close_fds=True)
        except ImportError:
            from popen2 import Popen3
            messages = Popen3(command)
            messages.stdout = messages.fromchild

        fcntl.fcntl(messages.stdout, fcntl.F_SETFL,
                    os.O_NONBLOCK | fcntl.fcntl(messages.stdout, fcntl.F_GETFL))

        def search(pattern):
            if isinstance(pattern, str):
                pattern = re.compile(pattern)
            content = b''
            timeout = 1.5
            while len(select.select([messages.stdout], [], [], timeout)[0]) > 0:
                timeout = 0.1
                added_content = messages.stdout.read()
                if not added_content:
                    break
                content += added_content
                if re.search(pattern, str(content)):
                    return
            message = \
                "expected: <%s> is found in <%s>\n" \
                " content: <%s>" % \
                (pp.format_re(pattern),
                 pp.format(log_file),
                 pp.format(content))
            self.fail(message)

        try:
            search(re.escape(mark))
            result = callable_object(*args, **kw_args)
            search(pattern)
        finally:
            os.kill(messages.pid, signal.SIGINT)
            messages.wait()

    def assert_exists(self, path):
        """
        Passes if path exists.

          self.assert_exists("/tmp/exist")        # => pass
          self.assert_exists("/tmp/nonexistence") # => fail
        """
        if os.path.exists(path):
            self._pass_assertion
        else:
            self.fail("expected: <%s> exists" % path)

    def assert_not_exists(self, path):
        """
        Passes if path doesn't exists.

          self.assert_not_exists("/tmp/nonexistence") # => pass
          self.assert_not_exists("/tmp/exist")        # => fail
        """
        if os.path.exists(path):
            self.fail("expected: <%s> doesn't exists" % path)
        else:
            self._pass_assertion

    def assert_open_file(self, name, *args):
        """
        Passes if open(name, *args) succeeds.

          file = self.assert_open_file("/tmp/exist", "w") # => pass
          self.assert_open_file("/tmp/nonexistence")      # => fail
        """
        try:
            result = open(name, *args)
        except IOError:
            exception_class, exception_value = sys.exc_info()[:2]
            message = \
                "expected: open(%s) succeeds\n" \
                " but was: <%s>(%s) is raised" % \
                (pp.format_call_arguments((name,) + args, {}),
                 pp.format_exception_class(exception_class),
                 str(exception_value))
            self._fail(message)
        self._pass_assertion
        return result

    def assert_try_call(self, timeout, interval,
                        callable_object, *args, **kw_args):
        """
        Passes if callable_object(*args, **kw_args) doesn't fail any
        assertions in <timeout> seconds.
        (It will tried <timeout / interval> times.)

          def random_number():
              number = random.randint(0, 9)
              self.assert_in_delta(5, number, 1)
              return number
          self.assert_try_call(1, 0.1, random_number) # => will pass
                                                      # returns 4, 5 or 6
          self.assert_try_call(1, 0.1, self.fail, "Never succeed") # => fail
        """
        rest = timeout
        while True:
            before = time.time()
            try:
                result = callable_object(*args, **kw_args)
                break
            except pikzie.core.AssertionFailure:
                if rest <= 0:
                    message = \
                        "expected: %s succeeds\n" \
                        " timeout: <%s> seconds\n" \
                        "interval: <%s> seconds\n" \
                        " but was:\n%s" % \
                        (pp.format_call(callable_object, args, kw_args),
                         timeout, interval,
                         str(sys.exc_info()[1]))
                    self._fail(message)
                runtime = time.time() - before
                wait_time = interval - runtime
                if wait_time > 0:
                    time.sleep(wait_time)
                rest -= max(runtime, interval)
        self._pass_assertion
        return result

    def assert_kernel_symbol(self, name):
        """
        Passes if /proc/kallsyms can be opened and name is in the list.

          self.assert_kernel_symbol("printk")       # => pass
                                                    # returns an address of printk
          self.assert_kernel_symbol("non_existent") # => fail
        """
        if not hasattr(os, "uname"):
            self.omit("only for Linux environment")
        if os.uname()[0] != "Linux":
            self.omit("only for Linux environment")

        for line in self.assert_open_file("/proc/kallsyms"):
            symbol_info = line.split()
            address = symbol_info[0]
            symbol_name = symbol_info[2]
            if name == symbol_name:
                self._pass_assertion
                return address
        self._fail("expected: <%r> is in kernel symbols" % name)



import sys
import re
import pikzie

import pikzie.pretty_print as pp

def collect_fault_info(fault):
    if hasattr(fault, "expected") and fault.expected and fault.actual:
        message = \
            ("expected: <%s>\n but was: <%s>") % \
            (fault.expected, fault.actual)
        if fault.message:
            message = fault.message + "\n" + message
    else:
        message = fault.message
    return (fault.symbol,
            str(fault.test),
            str(message),
            fault.test.metadata)

class Assertions(object):
    class RegexpMatchResult(object):
        def __init__(self, succeeded, test_result, fault_infos):
            self.succeeded = succeeded
            self.test_result = test_result
            self.fault_infos = list(fault_infos)

        def __eq__(self, other):
            if self.succeeded != other.succeeded: return False
            if self.test_result != other.test_result: return False
            if self.fault_infos == other.fault_infos: return True
            if len(self.fault_infos) != len(other.fault_infos): return False
            i = 0
            for fault_info in self.fault_infos:
                other_fault_info = other.fault_infos[i]
                if not self._regexp_support_string_equal(fault_info[0],
                                                         other_fault_info[0]):
                    return False
                if not self._regexp_support_string_equal(fault_info[1],
                                                         other_fault_info[1]):
                    return False
                if not self._regexp_support_string_equal(fault_info[2],
                                                         other_fault_info[2]):
                    return False
            return True

        def __repr__(self):
            def extract_pattern(pattern_or_string):
                if hasattr(pattern_or_string, "pattern"):
                    return pattern_or_string.pattern
                else:
                    return pattern_or_string
            patterns = [[extract_pattern(pattern_or_string)
                         for pattern_or_string
                         in fault_info]
                        for fault_info
                        in self.fault_infos]
            return str((self.succeeded, self.test_result, patterns))

        _pattern_class = type(re.compile(""))
        def _regexp_support_string_equal(self, value1, value2):
            if value1 == value2: return True
            if isinstance(value1, str) and isinstance(value2, str): return False
            if isinstance(value1, self._pattern_class):
                return re.search(value1, value2) is not None
            if isinstance(value2, self._pattern_class):
                return re.search(value2, value1) is not None
            return False

    def assert_result(self, succeeded, n_tests, n_assertions, n_failures,
                      n_errors, n_pendings, n_omissions, n_notifications,
                      fault_info, tests):
        context = pikzie.TestRunnerContext()
        for test_name in tests:
            self.TestCase(test_name).run(context)

            if context.need_interrupt():
                raise KeyboardInterrupt

        self.assert_equal(self.RegexpMatchResult(succeeded,
                                                 (n_tests,
                                                  n_assertions,
                                                  n_failures,
                                                  n_errors,
                                                  n_pendings,
                                                  n_omissions,
                                                  n_notifications),
                                                 fault_info),
                          self.RegexpMatchResult(context.succeeded,
                                                 (context.n_tests,
                                                  context.n_assertions,
                                                  context.n_failures,
                                                  context.n_errors,
                                                  context.n_pendings,
                                                  context.n_omissions,
                                                  context.n_notifications),
                                                 map(collect_fault_info,
                                                     context.faults)))

    def assert_success_output(self, n_tests, n_assertions, tests):
        self.assert_output("." * n_tests, n_tests, n_assertions, 0, 0, 0, 0, 0,
                           "", tests)

    def assert_output(self, progress, n_tests, n_assertions, n_failures,
                      n_errors, n_pendings, n_omissions, n_notifications,
                      details, tests):
        suite = pikzie.TestSuite(tests)
        context = self.runner.run(suite)
        self.assert_equal((n_tests, n_assertions, n_failures, n_errors,
                           n_pendings, n_omissions, n_notifications),
                          (context.n_tests, context.n_assertions,
                           context.n_failures, context.n_errors,
                           context.n_pendings, context.n_omissions,
                           context.n_notifications))
        self.output.seek(0)
        message = self.output.read()
        format = \
            "%s\n" \
            "%s" \
            "Finished in %.3f seconds\n" \
            "\n" \
            "%d test(s), %d assertion(s), %d failure(s), %d error(s), " \
            "%d pending(s), %d omission(s), %d notification(s)\n"
        self.assert_equal(format % (progress, details, context.elapsed,
                                    n_tests, n_assertions, n_failures,
                                    n_errors, n_pendings, n_omissions,
                                    n_notifications),
                          message)

class Source(object):
    def current_file(self):
        try:
            raise ZeroDivisionError
        except ZeroDivisionError:
            return sys.exc_info()[2].tb_frame.f_back.f_code.co_filename
    current_file = classmethod(current_file)

    def current_line_no(self):
        try:
            raise ZeroDivisionError
        except ZeroDivisionError:
            return sys.exc_info()[2].tb_frame.f_back.f_lineno
    current_line_no = classmethod(current_line_no)

    def find_target_line_no(self, pattern, file_name=None):
        if file_name is None:
            try:
                raise ZeroDivisionError
            except ZeroDivisionError:
                frame = sys.exc_info()[2].tb_frame.f_back
                file_name = frame.f_code.co_filename
        line_no = -1
        if type(pattern) == type(""):
            pattern = re.compile(re.escape(pattern))
        try:
            f = open(file_name)
            for i, line in enumerate(f):
                if pattern.search(line):
                    line_no = i + 1
                    break
        finally:
            f.close()
        return line_no
    find_target_line_no = classmethod(find_target_line_no)



import pikzie
import pikzie.module_base
from test.utils import Assertions, collect_fault_info

fixture_dir = "test/fixtures/module_based_test_cases"

def test_assertions():
    prefix = 'test_module_based_test_case_fixture'
    assert_result(True, 0, 0, 0, 0, 0, 0, [],
                  test_case_names=["/test_never_match/"])
    assert_result(False, 2, 2, 1, 0, 0, 0,
                  [['F',
                    prefix + '.test_assert_equal_fail',
                    "expected: <2>\n"
                    " but was: <1>",
                    None]],
                  test_case_names=["/test_module_based_test_case_fixture/"])

def assert_result(succeeded, n_tests, n_assertions, n_failures,
                  n_errors, n_pendings, n_notifications, fault_info,
                  **kw_args):
    context = pikzie.TestRunnerContext()
    _kw_args = {"base_dir": fixture_dir, "priority_mode": False}
    for name in kw_args:
        _kw_args[name] = kw_args[name]
    # _kw_args.update(**kw_args) # require Python >= 2.4
    loader = pikzie.TestLoader(**_kw_args)
    test_suite = loader.create_test_suite()
    test_suite.run(context)

    assert_equal(Assertions.RegexpMatchResult(succeeded,
                                              (n_tests,
                                               n_assertions,
                                               n_failures,
                                               n_errors,
                                               n_pendings,
                                               n_notifications),
                                              fault_info),
                 Assertions.RegexpMatchResult(context.succeeded,
                                              (context.n_tests,
                                               context.n_assertions,
                                               context.n_failures,
                                               context.n_errors,
                                               context.n_pendings,
                                               context.n_notifications),
                                              map(collect_fault_info,
                                                  context.faults)))



import pikzie

class TestXXX1(pikzie.TestCase):
    default_priority = "must"
    def no_test(self):
        pass

    def test_one(self):
        pass

    def test_two(self):
        pass

class TestXXX2(pikzie.TestCase):
    default_priority = "must"
    pass



#!/usr/bin/env python

import sys
import os

base_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
sys.path.insert(0, os.path.join(base_dir, "lib"))
sys.path.insert(0, base_dir)

import pikzie

sys.argv.insert(1, "--ignore-directory")
sys.argv.insert(2, "fixtures")
sys.exit(pikzie.Tester(version=pikzie.version).run())



# Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
import traceback
import os
import errno
import fnmatch
import types
import time
import tempfile

from pikzie.color import *
from pikzie.results import *
from pikzie.assertions import Assertions
from pikzie.decorators import metadata
from pikzie.priority import PriorityChecker

__all__ = ["TestSuite", "TestCase", "TestRunnerContext", "TestLoader"]

class TestSuite(object):
    """
    A test suite is a composite test consisting of a number of TestCases.

    For use, create an instance of TestSuite, then add test case instances.
    When all tests have been added, the suite can be passed to a test
    runner, such as TextTestRunner. It will run the individual test cases
    in the order in which they were added, aggregating the results. When
    subclassing, do not forget to call the base class constructor.
    """
    def __init__(self, tests=()):
        self._tests = []
        self.add_tests(tests)

    def __iter__(self):
        return iter(self._tests)

    def __len__(self):
        return sum(map(len, self._tests))

    def add_test(self, test):
        self._tests.append(test)

    def add_tests(self, tests):
        for test in tests:
            self.add_test(test)

    def run(self, context):
        context.on_start_test_suite(self)
        for test in self._tests:
            test.run(context)
            if context.need_interrupt():
                break
        context.on_finish_test_suite(self)

class TracebackEntry(object):
    def __init__(self, file_name, line_number, name, content):
        self.file_name = file_name
        self.line_number = line_number
        self.name = name
        self.content = content

    def __str__(self):
        result = '%s:%d: %s()' % (self.file_name, self.line_number, self.name)
        if self.content:
            result = "%s: %s" % (result, self.content)
        return result

class AssertionFailure(Exception):
    def __init__(self, message, user_message=None, expected=None, actual=None):
        self.message = message
        self.user_message = user_message
        self.expected = expected
        self.actual = actual

    def __str__(self):
        result = self.message
        if self.user_message:
            result = "%s\n%s" % (str(self.user_message).rstrip(), result)
        return result

class PendingTestError(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

class OmissionTestError(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

class TestCaseRunner(object):
    def __init__(self, test_case, tests, priority_mode=True):
        self.test_case = test_case
        self._tests = tests
        self.priority_mode = priority_mode

    def tests(self):
        tests = self._tests
        if self.priority_mode:
            tests = [test for test in tests if test.need_to_run()]
        return tests

    def run(self, context):
        tests = self.tests()
        if len(tests) == 0:
            return

        context.on_start_test_case(self.test_case)
        for test in tests:
            test.run(context)
            if context.need_interrupt():
                break
        context.on_finish_test_case(self.test_case)

class TestCaseTemplate(object):
    def setup(self):
        "Hook method for setting up the test fixture before exercising it."
        pass

    def teardown(self):
        "Hook method for deconstructing the test fixture after testing it."
        pass

class TestCase(TestCaseTemplate, Assertions):
    """
    A class whose instances are single test cases.

    If the fixture may be used for many test cases, create as
    many test methods as are needed. When instantiating such a TestCase
    subclass, specify in the constructor arguments the name of the test method
    that the instance is to execute.

    Test authors should subclass TestCase for their own tests. Construction
    and deconstruction of the test's environment ('fixture') can be
    implemented by overriding the 'setup' and 'teardown' methods respectively.

    If it is necessary to override the __init__ method, the base class
    __init__ method must always be called. It is important that subclasses
    should not change the signature of their __init__ method, since instances
    of the classes are instantiated automatically by parts of the framework
    in order to be run.
    """

    def _collect_test(cls, target, base_n_args):
        def is_function(object):
            return ((hasattr(object, "__code__") and
                     hasattr(object.__code__, "co_argcount")) or
                    (hasattr(object, "func_code") and
                     hasattr(object.func_code, "co_argcount")))
        def test_data(object):
            if not hasattr(object, metadata.container_key):
                return None
            return getattr(object, metadata.container_key).get("data")

        tests = []
        for name in dir(target):
            object = getattr(target, name)
            if not is_function(object):
                continue
            if hasattr(object, "__code__"):
                code = object.__code__
            else:
                code = object.func_code
            n_args = code.co_argcount
            data = test_data(object)
            if data is None:
                if n_args == base_n_args:
                    tests.append(cls(name))
            else:
                if not n_args == base_n_args + 1:
                    continue
                for datum in data:
                    tests.append(cls(name, datum["label"], datum["value"]))
        return tests
    _collect_test = classmethod(_collect_test)

    def collect_test(cls):
        return cls._collect_test(cls, 1)
    collect_test = classmethod(collect_test)

    def __init__(self, method_name, data_label=None, data=None):
        self.__method_name = method_name
        self.__description = self._test_method().__doc__
        self.__data_label = data_label
        self.__data = data

    def _data(self):
        return self.__data

    def _data_label(self):
        return self.__data_label

    def __len__(self):
        return 1

    def get_metadata(self, name):
        metadata_container = self.metadata
        if metadata_container is None:
            return None
        return metadata_container.get(name)

    def metadata(self):
        test_method = self._test_method()
        metadata_container_key = metadata.container_key
        if not hasattr(test_method, metadata_container_key):
            return None
        return getattr(test_method, metadata_container_key)
    metadata = property(metadata)

    def description(self):
        """
        Returns a one-line description of the test, or None if no
        description has been provided.

        The default implementation of this method returns the first line of
        the specified test method's docstring.
        """
        description = self.__description
        if description:
            return description.split("\n")[0].strip()
        else:
            return None

    def _test_case_name(self):
        return "%s.%s" % (self.__class__.__module__,
                          self.__class__.__name__)

    def id(self):
        id = "%s.%s" % (self._test_case_name(), self._method_name())
        if self.__data_label:
            id += " (%s)" % self.__data_label
        return id

    def __str__(self):
        string = "%s.%s" % (self.__class__.__name__, self._method_name())
        if self.__data_label:
            string += " (%s)" % self.__data_label
        return string

    def short_name(self):
        name = self.__method_name
        if self.__data_label:
            name += " (%s)" % self.__data_label
        return name

    def __repr__(self):
        return "<%s method_name=%s description=%s data_label=%s data=%s>" % \
            (str(self.__class__), self.__method_name, self.__description,
             self.__data_label, str(self.__data))

    def need_to_run(self):
        return not self._is_previous_test_success() or \
            self._need_to_run_according_to_priority()

    def run(self, context):
        success = False
        try:
            self._started(context)

            try:
                try:
                    self._run_setup(context)
                except PendingTestError:
                    self._pend_test(context)
                except OmissionTestError:
                    self._omit_test(context)
                except KeyboardInterrupt:
                    context.interrupt()
                    return
                except:
                    self._add_error(context)
                    return

                try:
                    self._run_test(context)
                    success = True
                except AssertionFailure:
                    self._add_failure(context)
                except PendingTestError:
                    self._pend_test(context)
                except OmissionTestError:
                    self._omit_test(context)
                except KeyboardInterrupt:
                    context.interrupt()
                    return
                except:
                    self._add_error(context)
            finally:
                try:
                    self._run_teardown(context)
                except PendingTestError:
                    self._pend_test(context)
                except OmissionTestError:
                    self._omit_test(context)
                except KeyboardInterrupt:
                    context.interrupt()
                except:
                    self._add_error(context)
                    success = False

        finally:
            self._finished(success, context)

    def _run_setup(self, context):
        self.setup()

    def _run_test(self, context):
        test_method = self._test_method()
        if self.__data_label:
            test_method(self.__data)
        else:
            test_method()

    def _run_teardown(self, context):
        self.teardown()

    def _method_name(self):
        return self.__method_name

    def _test_method(self):
        return getattr(self, self._method_name())

    def _pass_assertion(self):
        self.__context.pass_assertion(self)

    def _fail(self, message, user_message=None, expected=None, actual=None):
        raise AssertionFailure(message, user_message, expected, actual)

    def _pend(self, message):
        raise PendingTestError(message)

    def _omit(self, message):
        raise OmissionTestError(message)

    def _notify(self, message):
        try:
            raise ZeroDivisionError
        except ZeroDivisionError:
            frame = sys.exc_info()[2].tb_frame.f_back.f_back
        traceback = self._prepare_frame(frame, True)
        notification = Notification(self, message, traceback)
        self.__context.add_notification(self, notification)

    def _started(self, context):
        self.__context = context
        context.on_start_test(self)
        if os.path.exists(self._passed_file()):
            os.remove(self._passed_file())

    def _finished(self, success, context):
        if success:
            self._add_success(context)
        context.on_finish_test(self)
        self.__context = None

    def _add_success(self, context):
        open(self._passed_file(), "w").close()
        context.add_success(self)

    def _add_failure(self, context):
        exception_type, assertion_failure, traceback = sys.exc_info()
        traceback = self._prepare_traceback(traceback, True)
        failure = Failure(self, str(assertion_failure), traceback,
                          assertion_failure.expected,
                          assertion_failure.actual)
        context.add_failure(self, failure)

    def _add_error(self, context):
        exception_type, message, traceback = sys.exc_info()
        traceback = self._prepare_traceback(traceback, False)
        error = Error(self, exception_type, message, traceback)
        context.add_error(self, error)

    def _pend_test(self, context):
        exception_type, message, traceback = sys.exc_info()
        traceback = self._prepare_traceback(traceback, True)
        pending = Pending(self, message, traceback)
        context.pend_test(self, pending)

    def _omit_test(self, context):
        exception_type, message, traceback = sys.exc_info()
        traceback = self._prepare_traceback(traceback, True)
        omission = Omission(self, message, traceback)
        context.omit_test(self, omission)

    def _prepare_traceback(self, tb, compute_length):
        while tb and self._is_relevant_frame_level(tb.tb_frame):
            tb = tb.tb_next
        length = None
        if tb and compute_length:
            length = self._count_relevant_frame_levels(tb.tb_frame)
        stack_infos = traceback.extract_tb(tb, length)
        return self._create_traceback_entries(stack_infos)

    def _prepare_frame(self, frame, compute_length):
        while frame and self._is_relevant_frame_level(frame):
            frame = frame.f_back
        length = None
        if compute_length:
            length = self._count_relevant_frame_levels(frame)
        stack_infos = traceback.extract_stack(frame, length)
        return self._create_traceback_entries(stack_infos)

    def _create_traceback_entries(self, stack_infos):
        entries = []
        for stack_info in stack_infos:
            filename, lineno, name, line = stack_info
            entries.append(TracebackEntry(filename, lineno, name, line))
        return entries

    def _is_relevant_frame_level(self, frame):
        globals = frame.f_globals
        for cls in (TestCase,) + TestCase.__bases__:
            name = cls.__name__
            if name in globals and globals[name] == cls:
                return True
        return False

    def _count_relevant_frame_levels(self, frame):
        length = 0
        while frame and not self._is_relevant_frame_level(frame):
            length += 1
            frame = frame.f_back
        return length

    def _is_previous_test_success(self):
        return os.path.exists(self._passed_file())

    def _passed_file(self):
        return os.path.join(self._result_dir(), "passed")

    def _result_dir(self):
        components = [".test-result", self._test_case_name(), self.short_name()]
        parent_directories = [os.path.dirname(sys.argv[0]),
                              os.getcwd(),
                              os.path.join(os.path.dirname(__file__), "..")]
        if hasattr(os, "getuid"):
            parent_directories.append([os.path.join(tempfile.gettempdir(),
                                                    str(os.getuid()))])
        else:
            parent_directories.append([os.path.join(tempfile.gettempdir(),
                                                    str(os.getpid()))])
        for parent_directory in parent_directories:
            dir = os.path.abspath(os.path.join(parent_directory, *components))
            if os.path.isdir(dir):
                return dir
            try:
                os.makedirs(dir)
                return dir
            except OSError:
                pass

        raise OSError(errno.EACCES, "Permission denied",
                      ", ".join(parent_directories))

    default_priority = "normal"
    def _need_to_run_according_to_priority(self):
        priority = self.get_metadata("priority")
        if priority is None:
            priority = self.default_priority
        if hasattr(PriorityChecker, priority):
            return getattr(PriorityChecker, priority)()
        else:
            return True

class TestLoader(object):
    default_base_dir = os.path.dirname(sys.argv[0])
    default_pattern = "test[_-]*.py"
    default_ignore_dirs = [".svn", "CVS", ".git", ".test-result"]

    def __init__(self, base_dir=None, pattern=None, ignore_dirs=None,
                 test_names=None, test_case_names=None,
                 target_modules=None, priority_mode=True):
        self.base_dir = base_dir
        self.pattern = pattern
        self.ignore_dirs = ignore_dirs
        self.test_names = test_names
        self.test_case_names = test_case_names
        self.target_modules = target_modules or []
        self.priority_mode = priority_mode

    def _get_test_names(self):
        return self._test_names
    def _set_test_names(self, names):
        self._test_names = self._prepare_target_names(names)
    test_names = property(_get_test_names, _set_test_names)

    def _get_test_case_names(self):
        return self._test_case_names
    def _set_test_case_names(self, names):
        self._test_case_names = self._prepare_target_names(names)
    test_case_names = property(_get_test_case_names, _set_test_case_names)

    test_case_collectors = []

    _class_types = [type]
    if "ClassType" in types.__dict__:
        _class_types.append(types.ClassType)
    _class_types = tuple(_class_types)
    def _collect_test_case_from_module(self, module):
        test_cases = []
        for name in dir(module):
            object = getattr(module, name)
            if (isinstance(object, self._class_types) and
                issubclass(object, TestCase)):
                test_cases.append(object)
        return test_cases
    test_case_collectors.append(_collect_test_case_from_module)

    def collect_test_cases(self, files=[]):
        test_cases = []
        for module in self._load_modules(files):
            for test_case_collector in self.test_case_collectors:
                test_cases.extend(test_case_collector(self, module))

        def is_target_test_case_name(test_case):
            name = test_case.__name__
            if self.test_case_names is None:
                return True
            def is_target_name(test_case_name):
                if type(test_case_name) == str:
                    return test_case_name == name
                else:
                    return test_case_name.search(name)
            return len(list(filter(is_target_name, self.test_case_names))) > 0

        return list(filter(is_target_test_case_name, test_cases))

    def create_test_suite(self, files=[]):
        tests = []
        for test_case in self.collect_test_cases(files):
            def _is_target_test(test):
                return self._is_target_test(test)
            target_tests = filter(_is_target_test, test_case.collect_test())
            target_tests = list(target_tests)
            if len(target_tests) > 0:
                tests.append(TestCaseRunner(test_case, target_tests,
                                            self.priority_mode))
        return TestSuite(tests)

    def _find_targets(self):
        targets = []
        base_dir = os.path.abspath(self.base_dir or self.default_base_dir)
        pattern = self.pattern or self.default_pattern
        ignore_dirs = (self.ignore_dirs or []) + self.default_ignore_dirs
        for root, dirs, files in os.walk(base_dir):
            for file in files:
                if fnmatch.fnmatch(file, pattern):
                    targets.append(re.sub("^\./", "", os.path.join(root, file)))
            for ignore_dir in ignore_dirs:
                if ignore_dir in dirs:
                    dirs.remove(ignore_dir)
        return (base_dir,
                map(lambda path: re.sub("^%s\/" % re.escape(base_dir), "", path),
                    targets))

    def _need_load_files(self, files, modules):
        if self.pattern is not None:
            return True
        if len(files) == 0 and len(modules) == 0:
            return True
        return False

    def _load_modules(self, files=[]):
        modules = self.target_modules[:]
        targets = files[:]
        base_dir = None
        if self._need_load_files(files, modules):
            base_dir, _targets = self._find_targets()
            targets += _targets
        if base_dir:
            base_dir = os.path.abspath(base_dir)
            sys.path.insert(0, base_dir)
        for target in targets:
            target = os.path.splitext(target)[0]
            target = re.sub(re.escape(os.path.sep), ".", target)
            parts = target.split(".")
            module = None
            while len(parts) > 0 and module is None:
                name = ".".join(parts)
                __import__(name)
                module = sys.modules[name]
                parts.pop()
            if module is not None and module not in modules:
                modules.append(module)
        if base_dir:
            sys.path.remove(base_dir)
        return modules

    def _is_target_test(self, test):
        name = test.short_name()
        if not name.startswith("test_"):
            return False
        if self.test_names is not None:
            def is_target_name(test_name):
                if type(test_name) == str:
                    return test_name == name
                else:
                    return test_name.search(name)
            if len(list(filter(is_target_name, self.test_names))) == 0:
                return False
        return True

    def _prepare_target_names(self, names):
        if names is None: return names
        if type(names) == str:
            names = [names]
        re_re = re.compile("/(.*)/([ilmsux]*)")
        def prepare(name):
            match = re_re.search(name)
            if match:
                flags = 0
                for flag_string in match.groups()[1]:
                    flags |= getattr(re, flag_string.upper())
                name = re.compile(match.groups()[0], flags)
            return name
        return [prepare(name) for name in names]

class TestRunnerContext(object):
    """
    Context for running test.

    Test results are automatically managed by the TestCase and TestSuite
    classes, and do not need to be explicitly manipulated by writers of tests.

    Each instance holds the total number of tests run, and collections of
    failures and errors that occurred among those test runs. The collections
    contain tuples of (testcase, exceptioninfo), where exceptioninfo is the
    formatted traceback of the error that occurred.
    """
    def __init__(self):
        self.n_assertions = 0
        self.n_tests = 0
        self.results = []
        self.listeners = []
        self.interrupted = False
        self.elapsed = 0

    def add_listener(self, listener):
        self.listeners.append(listener)

    def add_listeners(self, listeners):
        self.listeners.extend(listeners)

    def faults(self):
        return list(filter(lambda result: result.fault, self.results))
    faults = property(faults)

    def n_faults(self):
        return len(self.faults)
    n_faults = property(n_faults)

    def n_failures(self):
        return len(list(filter(lambda result: isinstance(result, Failure),
                               self.results)))
    n_failures = property(n_failures)

    def n_errors(self):
        return len(list(filter(lambda result: isinstance(result, Error),
                               self.results)))
    n_errors = property(n_errors)

    def n_pendings(self):
        return len(list(filter(lambda result: isinstance(result, Pending),
                               self.results)))
    n_pendings = property(n_pendings)

    def n_omissions(self):
        return len(list(filter(lambda result: isinstance(result, Omission),
                               self.results)))
    n_omissions = property(n_omissions)

    def n_notifications(self):
        return len(list(filter(lambda result: isinstance(result, Notification),
                               self.results)))
    n_notifications = property(n_notifications)

    def pass_assertion(self, test):
        self.n_assertions += 1
        self._notify("pass_assertion", test)

    def on_start_test(self, test):
        "Called when the given test is about to be run"
        self._start_at = time.time()
        self.n_tests += 1
        self._notify("start_test", test)

    def on_finish_test(self, test):
        "Called when the given test has been run"
        self._finish_at = time.time()
        self.elapsed += (self._finish_at - self._start_at)
        self._notify("finish_test", test)

    def on_start_test_case(self, test_case):
        "Called when the given test case is about to be run"
        self._notify("start_test_case", test_case)

    def on_finish_test_case(self, test_case):
        "Called when the given test case has been run"
        self._notify("finish_test_case", test_case)

    def on_start_test_suite(self, test_suite):
        "Called when the given test suite is about to be run"
        self._notify("start_test_suite", test_suite)

    def on_finish_test_suite(self, test_suite):
        "Called when the given test suite has been run"
        self._notify("finish_test_suite", test_suite)

    def add_error(self, test, error):
        """Called when an error has occurred."""
        error.elapsed = time.time() - self._start_at
        self.results.append(error)
        self._notify("error", error)

    def add_failure(self, test, failure):
        """Called when a failure has occurred."""
        failure.elapsed = time.time() - self._start_at
        self.results.append(failure)
        self._notify("failure", failure)

    def add_notification(self, test, notification):
        """Called when a notification has occurred."""
        notification.elapsed = time.time() - self._start_at
        self.results.append(notification)
        self._notify("notification", notification)

    def add_success(self, test):
        "Called when a test has completed successfully"
        success = Success(test)
        success.elapsed = time.time() - self._start_at
        self.results.append(success)
        self._notify("success", success)

    def pend_test(self, test, pending):
        """Called when a test is pended."""
        pending.elapsed = time.time() - self._start_at
        self.results.append(pending)
        self._notify("pending", pending)

    def omit_test(self, test, omission):
        """Called when a test is omitted."""
        omission.elapsed = time.time() - self._start_at
        self.results.append(omission)
        self._notify("omission", omission)

    def interrupt(self):
        "Indicates that the tests should be interrupted"
        self.interrupted = True

    def need_interrupt(self):
        return self.interrupted

    def succeeded(self):
        return len(list(filter(lambda fault: fault.critical, self.faults))) == 0
    succeeded = property(succeeded)

    def _notify(self, name, *args):
        for listener in self.listeners:
            callback_name = "on_%s" % name
            if hasattr(listener, callback_name):
                getattr(listener, callback_name)(self, *args)

    def summary(self):
        return ("%d test(s), %d assertion(s), %d failure(s), %d error(s), " \
                    "%d pending(s), %d omission(s), %d notification(s)") % \
            (self.n_tests, self.n_assertions, self.n_failures, self.n_errors,
             self.n_pendings, self.n_omissions, self.n_notifications)

    __str__ = summary



# Copyright (C) 2009-2016  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

version = "1.0.3"

try:
    import __builtin__
except ImportError:
    pass
else:
    if not hasattr(__builtin__, "sorted"):
        def sorted(iterable, cmd=None, key=None, reverse=False):
            list = iterable[:]
            list.sort(cmd)
            if reverse:
                list.reverse()
                return list
        __builtin__.sorted = sorted

from pikzie.tester import Tester
from pikzie.core import *
from pikzie.decorators import *
from pikzie.module_base import *
from pikzie.utils import *



# Copyright (C) 2010  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pikzie
from pikzie.utils import *

tmp_dir = os.path.join(os.path.dirname(__file__), "tmp")

def setup():
    rm_rf(tmp_dir)

def teardown():
    rm_rf(tmp_dir)

def test_mkdir_p():
    deep_dir = os.path.join(tmp_dir, "deep", "dir")
    assert_not_exists(deep_dir)
    mkdir_p(deep_dir)
    assert_exists(deep_dir)

def test_rm_rf():
    deep_dir = os.path.join(tmp_dir, "deep", "dir")
    deep_file = os.path.join(deep_dir, "file")
    mkdir_p(deep_dir)
    assert_open_file(deep_file, "w")
    assert_exists(tmp_dir)
    rm_rf(tmp_dir)
    assert_not_exists(tmp_dir)

def test_cp_a():
    source_dir = os.path.join(tmp_dir, "src")
    shallow_copy = os.path.join(source_dir, "file")
    deep_dir = os.path.join(source_dir, "deep", "dir")
    deep_file = os.path.join(deep_dir, "file")
    mkdir_p(deep_dir)
    assert_open_file(shallow_copy, "w")
    assert_open_file(deep_file, "w")

    destination_dir = os.path.join(tmp_dir, "dest")
    assert_not_exists(destination_dir)
    cp_a(source_dir, destination_dir)
    assert_exists(destination_dir)
    tree = []
    for root, dirs, files in os.walk(destination_dir):
        for base in dirs + files:
            tree.append(os.path.join(root, base))
    assert_equal(sorted([os.path.join(destination_dir, "file"),
                         os.path.join(destination_dir, "deep"),
                         os.path.join(destination_dir, "deep", "dir"),
                         os.path.join(destination_dir, "deep", "dir", "file")]),
                 sorted(tree))



import pikzie

def test_assert_equal(data):
    assert_equal(data["expected"], data["actual"])
test_assert_equal = \
    pikzie.data("success",
                {"expected": "abc", "actual": "abc"})(test_assert_equal)
test_assert_equal = \
    pikzie.data("fail",
                {"expected": "abc", "actual": "def"})(test_assert_equal)
test_assert_equal = \
    pikzie.data("another-success",
                {"expected": "def", "actual": "def"})(test_assert_equal)



import re

import pikzie
import test.utils

color = pikzie.color.Color
mix_color = pikzie.color.MixColor

class TestColor(pikzie.TestCase, test.utils.Assertions):
    """Tests for color."""

    def test_color_escape_sequence(self):
        self.assert_escape_sequence(["31"], color("red"))
        self.assert_escape_sequence(["32", "1"], color("green", bold=True))
        self.assert_escape_sequence(["0"], color("reset"))
        self.assert_escape_sequence(["45"], color("magenta", foreground=False))

    def test_mix_color_escape_sequence(self):
        self.assert_escape_sequence(["34", "1"],
                                    mix_color([color("blue"),
                                               color("none", bold=True)]))
        self.assert_escape_sequence(["34", "1", "4"],
                                    mix_color([color("blue"),
                                               color("none", bold=True)]) + \
                                        color("none", underline=True))

    def assert_escape_sequence(self, expected, color):
        self.assert_equal(expected, color.sequence)
        self.assert_match("\033\[(?:\\d+;)*\\d+m", color.escape_sequence)
        self.assert_equal(expected, color.escape_sequence[2:-1].split(";"))



# Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

from pikzie.core import *
from pikzie.assertions import Assertions

__all__ = []

__current_test_case__ = None
current_module = sys.modules[__name__]
assertions = {}
for assertion in filter(lambda name: not name.startswith("_"), dir(Assertions)):
    def wrap_assertion(assertion):
        _assertion = getattr(Assertions, assertion)
        def run_assertion(*args, **kw_args):
            if __current_test_case__ is None:
                def inspect_kw_arg(arg):
                    return arg + ("=%s" % kw_args[arg])
                inspected_args = ", ".join(map(str, args) +
                                           map(inspect_kw_arg, kw_args))
                raise TypeError("did you mean: self.%s(%s)" % \
                                    (assertion, inspected_args))
            return _assertion(__current_test_case__, *args, **kw_args)
        return run_assertion
    wrapped_assertion = wrap_assertion(assertion)
    assertions[assertion] = wrapped_assertion
    current_module.__dict__[assertion] = wrapped_assertion
    __all__.append(assertion)

def collect_test_case_from_module(self, module):
    test_cases = []
    pikzie_module = sys.modules.get("pikzie")
    if (pikzie_module is None):
        return []
    module_object = type(pikzie_module)
    def _is_module(_object):
        return type(_object) == module_object
    if (pikzie_module in filter(_is_module,
                                map(lambda name: getattr(module, name),
                                    dir(module)))):
        for assertion in assertions:
            if not assertion in module.__dict__:
                module.__dict__[assertion] = assertions[assertion]
        test_cases.append(type(module.__name__,
                               (ModuleBasedTestCase,),
                               {"target_module": module}))
    return test_cases

TestLoader.test_case_collectors.append(collect_test_case_from_module)

class ModuleBasedTestCase(TestCase):
    __current_test_case__ = None

    def collect_test(cls):
        return cls._collect_test(cls.target_module, 0)
    collect_test = classmethod(collect_test)

    def _test_method(self):
        return getattr(self.__class__.target_module, self._method_name())

    def _test_case_name(self):
        return self.__class__.target_module.__name__

    def __str__(self):
        return self.id()

    def __repr__(self):
        return "<%s method_name=%s description=%s data_label=%s data=%s>" % \
            (str(self.__class__.target_module),
             self._method_name(), self.__description,
             self.__data_label, str(self.__data))

    def _run_setup(self, context):
        setup = getattr(self.__class__.target_module, "setup", None)
        if setup:
            setup()

    def _run_teardown(self, context):
        teardown = getattr(self.__class__.target_module, "teardown", None)
        if teardown:
            teardown()

    def _run_test(self, context):
        global __current_test_case__
        before_test_case = __current_test_case__
        try:
            __current_test_case__ = self
            return TestCase._run_test(self, context)
        finally:
            __current_test_case__ = before_test_case



# Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ["metadata", "bug", "priority", "data"]

def override_setter(container, name, value):
    container[name] = value

def append_setter(container, name, value):
    if container.get(name) is None:
        container[name] = []
    container[name].append(value)

def metadata(name, value, setter=None):
    """Set metadata to a method."""
    if setter is None:
        setter = override_setter
    def decorator(function):
        if not hasattr(function, metadata.container_key):
            setattr(function, metadata.container_key, {})
        setter(getattr(function, metadata.container_key), name, value)
        return function
    return decorator
metadata.container_key = "__metadata__"

def bug(id):
    """Set Bug ID to a method."""
    return metadata("bug", id)

def priority(priority):
    """Set priority of test."""
    return metadata("priority", priority)

def data(label, value):
    """Set test data."""
    return metadata("data", {"label": label, "value": value}, append_setter)



import re

try:
    from exceptions import *
except ImportError:
    pass

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import pikzie
from pikzie.ui.console import ConsoleTestRunner
import pikzie.pretty_print as pp

from test.utils import *

class TestRunner(pikzie.TestCase, Assertions):
    def setup(self):
        self.output = StringIO()
        self.runner = ConsoleTestRunner(self.output, use_color=False)
        self.file_name = __file__
        if self.file_name.endswith("pyc"):
            self.file_name = self.file_name[:-1]

    def test_run_empty_test(self):
        class TestCase(pikzie.TestCase):
            def test_nothing(self):
                pass
        test = TestCase("test_nothing")
        self.assert_success_output(1, 0, [test])

    def test_run_test_with_assertions(self):
        class TestCase(pikzie.TestCase):
            def test_assertion(self):
                self.assert_equal(3, 1 + 2)
                self.assert_equal("aaaaa", "a" * 5)

        test = TestCase("test_assertion")
        self.assert_success_output(1, 2, [test])

    def test_run_failed_test(self):
        class TestCase(pikzie.TestCase):
            def test_fail_assertion(self):
                self.assert_equal(3, 1 + 2)
                self.assert_equal("aaaaa", "a")

        test = TestCase("test_fail_assertion")
        format = \
            "\n" \
            "1) Failure: TestCase.test_fail_assertion: %s\n" \
            "%s:%d: %s\n" \
            "expected: <'aaaaa'>\n" \
            " but was: <'a'>\n" \
            "\n"
        target_line = "self.assert_equal(\"aaaaa\", \"a\")"
        line_no = Source.find_target_line_no(target_line)
        details = format % (target_line, self.file_name, line_no, target_line)
        self.assert_output("F", 1, 1, 1, 0, 0, 0, 0, details, [test])

    def test_run_error_test(self):
        class TestCase(pikzie.TestCase):
            def test_error_raised(self):
                self.unknown_method(12345)
                self.assert_equal(3, 1 + 2)

        test = TestCase("test_error_raised")
        format = \
            "\n" \
            "1) Error: TestCase.test_error_raised\n" \
            "%s:%d: %s\n" \
            "%s: " \
            "'TestCase' object has no attribute 'unknown_method'\n" \
            "\n"
        target_line = "self.unknown_method(12345)"
        line_no = Source.find_target_line_no(target_line)
        details = format % (self.file_name, line_no, target_line,
                            AttributeError)
        self.assert_output("E", 1, 0, 0, 1, 0, 0, 0, details, [test])

    def test_metadata(self):
        class TestCase(pikzie.TestCase):
            def test_error_raised(self):
                self.unknown_attribute
                self.assert_equal(3, 1 + 2)
            test_error_raised = pikzie.bug(123)(test_error_raised)

            def test_with_metadata(self):
                self.assert_equal(3, 1 - 2)
            test_with_metadata = pikzie.bug(999)(test_with_metadata)
            test_with_metadata = \
                pikzie.metadata("key", "value")(test_with_metadata)

        tests = [TestCase("test_error_raised"),
                 TestCase("test_with_metadata")]
        format = \
            "\n" \
            "1) Error: TestCase.test_error_raised\n" \
            "  bug: 123\n" \
            "%s:%d: %s\n" \
            "%s: " \
            "'TestCase' object has no attribute 'unknown_attribute'\n" \
            "\n" \
            "2) Failure: TestCase.test_with_metadata: %s\n" \
            "  bug: 999\n" \
            "  key: value\n" \
            "%s:%d: %s\n" \
            "expected: <3>\n" \
            " but was: <-1>\n" \
            "\n"
        target_line1 = "self.unknown_attribute"
        line_no1 = Source.find_target_line_no(target_line1)
        target_line2 = "self.assert_equal(3, 1 - 2)"
        line_no2 = Source.find_target_line_no(target_line2)
        details = format % (self.file_name, line_no1, target_line1,
                            AttributeError,
                            target_line2, self.file_name, line_no2, target_line2)
        self.assert_output("EF", 2, 0, 1, 1, 0, 0, 0, details, tests)

    def test_pending(self):
        class TestCase(pikzie.TestCase):
            def test_pend(self):
                self.pend("just a minute!")

        test = TestCase("test_pend")
        format = \
            "\n" \
            "1) Pending: TestCase.test_pend: just a minute!\n" \
            "%s:%d: %s\n" \
            "\n"
        target_line = "self.pend(\"just a minute!\")"
        line_no = Source.find_target_line_no(target_line)
        details = format % (self.file_name, line_no, target_line)
        self.assert_output("P", 1, 0, 0, 0, 1, 0, 0, details, [test])

    def test_omission(self):
        class TestCase(pikzie.TestCase):
            def test_omit(self):
                self.omit("just a minute!")

        test = TestCase("test_omit")
        format = \
            "\n" \
            "1) Omission: TestCase.test_omit: just a minute!\n" \
            "%s:%d: %s\n" \
            "\n"
        target_line = "self.omit(\"just a minute!\")"
        line_no = Source.find_target_line_no(target_line)
        details = format % (self.file_name, line_no, target_line)
        self.assert_output("O", 1, 0, 0, 0, 0, 1, 0, details, [test])

    def test_notification(self):
        class TestCase(pikzie.TestCase):
            def test_notify(self):
                self.assert_equal(1, 3 - 2)
                self.notify("Call me!")
                self.assert_equal(5, 3 + 2)

        test = TestCase("test_notify")
        format = \
            "\n" \
            "1) Notification: TestCase.test_notify: Call me!\n" \
            "%s:%d: %s\n" \
            "\n"
        target_line = "self.notify(\"Call me!\")"
        line_no = Source.find_target_line_no(target_line)
        details = format % (self.file_name, line_no, target_line)
        self.assert_output("N.", 1, 2, 0, 0, 0, 0, 1, details, [test])

    def test_run_failed_tuple_data(self):
        class TestCase(pikzie.TestCase):
            def test_fail_assertion_tuple_data(self, data):
                self.assert_equal(data, data)
                self.assert_equal("tuple", data)
        label = "fail"
        data = (1, 2)
        test = TestCase("test_fail_assertion_tuple_data", label, data)
        format = \
            "\n" \
            "1) Failure: TestCase.test_fail_assertion_tuple_data (%s): %s\n" \
            "  data: %s\n" \
            "%s:%d: %s\n" \
            "expected: <'tuple'>\n" \
            " but was: <%s>\n" \
            "\n"
        target_line = "self.assert_equal(\"tuple\", data)"
        line_no = Source.find_target_line_no(target_line)
        details = format % (label, target_line, str(data),
                            self.file_name, line_no, target_line, str(data))
        self.assert_output("F", 1, 1, 1, 0, 0, 0, 0, details, [test])

    def test_run_failed_list_data(self):
        class TestCase(pikzie.TestCase):
            def test_fail_assertion_list_data(self, data):
                self.assert_equal(data, data)
                self.assert_equal("list", data)
        label = "fail"
        data = [1, 2]
        test = TestCase("test_fail_assertion_list_data", label, data)
        format = \
            "\n" \
            "1) Failure: TestCase.test_fail_assertion_list_data (%s): %s\n" \
            "  data: %s\n" \
            "%s:%d: %s\n" \
            "expected: <'list'>\n" \
            " but was: <%s>\n" \
            "\n"
        target_line = "self.assert_equal(\"list\", data)"
        line_no = Source.find_target_line_no(target_line)
        details = format % (label, target_line, str(data),
                            self.file_name, line_no, target_line, str(data))
        self.assert_output("F", 1, 1, 1, 0, 0, 0, 0, details, [test])

    def test_run_failed_dict_data(self):
        class TestCase(pikzie.TestCase):
            def test_fail_assertion_dict_data(self, data):
                self.assert_equal(data, data)
                self.assert_equal("dict", data)
        label = "fail"
        data = {"a": 1, "b": 2}
        test = TestCase("test_fail_assertion_dict_data", label, data)
        format = \
            "\n" \
            "1) Failure: TestCase.test_fail_assertion_dict_data (%s): %s\n" \
            "  data: %s\n" \
            "%s:%d: %s\n" \
            "expected: <'dict'>\n" \
            " but was: <%s>\n" \
            "\n"
        target_line = "self.assert_equal(\"dict\", data)"
        line_no = Source.find_target_line_no(target_line)
        details = format % (label, target_line, str(data),
                            self.file_name, line_no, target_line,
                            pp.format(data))
        self.assert_output("F", 1, 1, 1, 0, 0, 0, 0, details, [test])




# Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import cgi

class XML(object):
    def __init__(self, output):
        self.file = isinstance(output, str)
        if self.file:
            output = open(output, "w")
        self.output = output
        self.have_test = False

    def on_start_test(self, report, test):
        if not self.have_test:
            self.have_test = True
            self._write("<report>\n")

    def _on_result(self, report, result):
        self._write_result(result)

    on_success = _on_result
    on_failure = _on_result
    on_error = _on_result
    on_pending = _on_result
    on_notification = _on_result

    def on_finish_test_suite(self, report, test_suite):
        if self.have_test:
            self._write("</report>")
        else:
            self._write("<report/>")
        self._write("\n")
        if self.file:
            self.output.close()

    try:
        unicode
        def _normalize(self, string):
            return unicode(string)
    except:
        def _normalize(self, string):
            return str(string)

    def _write(self, string):
        self.output.write(self._normalize(string))

    def _write_tag(self, indent, name, content):
        if content:
            self._write("%s<%s>%s</%s>\n" % (indent, name,
                                             cgi.escape(str(content)),
                                             name))
        else:
            self._write("%s<%s/>\n" % (indent, name))

    def _write_result(self, result):
        self._write("  <result>\n")
        self._write_test_case(result.test.__class__)
        self._write_test(result.test)
        self._write_tag("    ", "status", result.name)
        self._write_tag("    ", "detail", result.detail())
        self._write_tag("    ", "elapsed", "%f" % result.elapsed)
        self._write_traceback(result.traceback)
        self._write("  </result>\n")

    def _write_test_case(self, test_case):
        name = "%s.%s" % (test_case.__module__, test_case.__name__)
        description = test_case.__doc__
        self._write("    <test_case>\n")
        self._write_tag("      ", "name", name)
        self._write_tag("      ", "description", description)
        self._write("    </test_case>\n")

    def _write_test(self, test):
        self._write("    <test>\n")
        self._write_tag("      ", "name", test.short_name())
        self._write_tag("      ", "description", test.description())
        self._write_test_metadata(test.metadata)
        self._write("    </test>\n")

    def _write_test_metadata(self, metadata):
        if not metadata:
            return
        for key in metadata:
            self._write("      <option>\n")
            self._write_tag("        ", "name", key)
            self._write_tag("        ", "value", metadata[key])
            self._write("      </option>\n")

    def _write_traceback(self, traceback):
        if not traceback:
            return
        self._write("    <backtrace>\n")
        self._write("      <entry>\n")
        for entry in traceback:
            self._write_tag("        ", "file", entry.file_name)
            self._write_tag("        ", "line", entry.line_number)
            self._write_tag("        ", "info", entry.content)
        self._write("      </entry>\n")
        self._write("    </backtrace>\n")



# Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import random

class PriorityChecker(object):
    def must():
        return True
    must = staticmethod(must)

    def important():
        return random.random() >= 0.1
    important = staticmethod(important)

    def high():
        return random.random() >= 0.3
    high = staticmethod(high)

    def normal():
        return random.random() >= 0.5
    normal = staticmethod(normal)

    def low():
        return random.random() >= 0.75
    low = staticmethod(low)

    def never():
        return False
    never = staticmethod(never)



# Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import atexit
from optparse import OptionParser

from pikzie.core import *
from pikzie.ui.console import *
import pikzie.report

class Tester(object):
    """
    A command-line program that runs a set of tests; this is primarily
    for making test modules conveniently executable.
    """

    ran = False

    def __init__(self, version=None, target_modules=None):
        self.version = version
        if target_modules is not None:
            def ensure_module(module_or_name):
                if type(module_or_name) == str:
                    return sys.modules[module_or_name]
                else:
                    return module_or_name
            target_modules = list(map(ensure_module, target_modules))
        self.target_modules = target_modules

    def run(self, args=None):
        self.__class__.ran = True
        options, args = self._parse(args)
        options = options.__dict__
        test_suite_create_options = {
            "base_dir": options.pop("base_dir"),
            "ignore_dirs": options.pop("ignore_dirs"),
            "pattern": options.pop("test_file_name_pattern"),
            "test_names": options.pop("test_names"),
            "test_case_names": options.pop("test_case_names"),
            "target_modules": self.target_modules,
            "priority_mode": options.pop("priority_mode")
        }
        xml_report = options.pop("xml_report")
        test = TestLoader(**test_suite_create_options).create_test_suite(args)
        runner = ConsoleTestRunner(**options)
        listeners = []
        if xml_report:
            listeners.append(pikzie.report.XML(xml_report))
        context = runner.run(test, listeners)
        if context.succeeded:
            return 0
        else:
            return 1

    def _parse(self, args):
        parser = OptionParser(version=self.version,
                              usage="%prog [options] [test_files]")
        group = parser.add_option_group("Common", "Common options")
        group.add_option("--base-dir",
                         metavar="DIR", dest="base_dir",
                         help="Base directory that has tests "
                         "(default: %s)" % TestLoader.default_base_dir)
        group.add_option("--ignore-directory", metavar="DIRECTORY",
                         action="append", dest="ignore_dirs",
                         help="Don't load tests under DIRECTORY "
                         "(default: %r)" % TestLoader.default_ignore_dirs)
        group.add_option("-p", "--test-file-name-pattern",
                         metavar="PATTERN", dest="test_file_name_pattern",
                         help="Glob for test file base name "
                         "(default: %s)" % TestLoader.default_pattern)
        group.add_option("-n", "--name", metavar="TEST_NAME",
                         action="append", dest="test_names",
                         help="Specify tests")
        group.add_option("-t", "--test-case", metavar="TEST_CASE_NAME",
                         action="append", dest="test_case_names",
                         help="Specify test cases")
        group.add_option("--xml-report", metavar="FILE",
                         dest="xml_report",
                         help="Report test result to FILE as XML")
        group.add_option("--priority", action="store_true", default=False,
                         dest="priority_mode", help="Use priority mode")
        group.add_option("--no-priority", action="store_false",
                         dest="priority_mode", help="Not use priority mode")
        ConsoleTestRunner.setup_options(parser)
        return parser.parse_args(args)

auto_test_run_reject_pattern = \
    r"\b(?:pydoc[\d.]*|setup\.py|ipython[\d.]*|easy_install[\d.]*)$"

def auto_test_run():
    if Tester.ran: return
    if not sys.argv[0]: return
    if sys.argv[0] == '-c': return
    if re.search(auto_test_run_reject_pattern, sys.argv[0]): return
    sys.exit(Tester(target_modules=['__main__']).run())

atexit.register(auto_test_run)



import re
import cgi

try:
    from exceptions import *
except ImportError:
    pass

try:
    from io import StringIO
except ImportError:
    from StringIO import StringIO

import pikzie
import pikzie.report
import pikzie.ui.console
from test.utils import *

ConsoleTestRunner = pikzie.ui.console.ConsoleTestRunner
VERBOSE_LEVEL_SILENT = pikzie.ui.console.VERBOSE_LEVEL_SILENT

class TestXMLReport(pikzie.TestCase, Assertions):
    """Tests for XML report."""

    class TestCase(pikzie.TestCase, Assertions):
        """A test case for test"""

        def _test_success(self):
            self.assert_true(True)

        TEST_FAILURE_LINE = Source.current_line_no() + 2
        def _test_failure(self):
            self.assert_true(False)
        _test_failure = pikzie.bug(1234)(_test_failure)

        TEST_ERROR_LINE = Source.current_line_no() + 3
        def _test_error(self):
            """Should error!!!"""
            self.non_existence_method()

    def test_empty_test(self):
        self.assert_xml("<report/>\n", self._suite())

    def test_success_result(self):
        elapsed = "0.001"
        xml = """
<report>
  <result>
    <test_case>
      <name>test_xml_report.TestCase</name>
      <description>A test case for test</description>
    </test_case>
    <test>
      <name>_test_success</name>
      <description/>
    </test>
    <status>success</status>
    <detail/>
    <elapsed>%s</elapsed>
  </result>
</report>
"""
        xml = (xml.strip() + "\n") % elapsed
        self.assert_xml(xml, self._suite(["_test_success"]), elapsed)

    def test_failure_result(self):
        elapsed = "0.001"
        xml = """
<report>
  <result>
    <test_case>
      <name>test_xml_report.TestCase</name>
      <description>A test case for test</description>
    </test_case>
    <test>
      <name>_test_failure</name>
      <description/>
      <option>
        <name>bug</name>
        <value>1234</value>
      </option>
    </test>
    <status>failure</status>
    <detail>expected: &lt;False&gt; is a true value</detail>
    <elapsed>%s</elapsed>
    <backtrace>
      <entry>
        <file>%s</file>
        <line>%s</line>
        <info>self.assert_true(False)</info>
      </entry>
    </backtrace>
  </result>
</report>
"""
        xml = xml % (elapsed,
                     Source.current_file(),
                     self.TestCase.TEST_FAILURE_LINE)
        xml = xml.strip() + "\n"
        self.assert_xml(xml, self._suite(["_test_failure"]), elapsed)

    def test_error_result(self):
        elapsed = "0.001"
        xml = """
<report>
  <result>
    <test_case>
      <name>test_xml_report.TestCase</name>
      <description>A test case for test</description>
    </test_case>
    <test>
      <name>_test_error</name>
      <description>Should error!!!</description>
    </test>
    <status>error</status>
    <detail>%s</detail>
    <elapsed>%s</elapsed>
    <backtrace>
      <entry>
        <file>%s</file>
        <line>%s</line>
        <info>self.non_existence_method()</info>
      </entry>
    </backtrace>
  </result>
</report>
"""
        xml = xml % (("%s: 'TestCase' object has no attribute " +
                      "'non_existence_method'") % \
                         cgi.escape(str(AttributeError)),
                     elapsed,
                     Source.current_file(),
                     self.TestCase.TEST_ERROR_LINE)
        xml = xml.strip() + "\n"
        self.assert_xml(xml, self._suite(["_test_error"]), elapsed)

    def _suite(self, names=[]):
        return pikzie.TestSuite([self.TestCase(name) for name in names])

    TEST_RUN_LINE = Source.current_line_no() + 3
    def assert_xml(self, expected, suite, normalized_elapsed=None):
        runner = ConsoleTestRunner(verbose_level=VERBOSE_LEVEL_SILENT)
        report_output = StringIO()
        report = pikzie.report.XML(report_output)
        result = runner.run(suite, [report])
        report_output.seek(0)
        xml = report_output.read()
        if xml and normalized_elapsed:
            xml = re.sub(r"<elapsed>[\d.]+<\/elapsed>",
                         "<elapsed>%s</elapsed>" % normalized_elapsed,
                         xml)
        self.assert_equal(expected, xml)



# Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

class Color(object):
    names = ["black", "red", "green", "yellow",
             "blue", "magenta", "cyan", "white"]
    def __init__(self, name, foreground=True, intensity=False,
                 bold=False, italic=False, underline=False):
        self.name = name
        self.foreground = foreground
        self.intensity = intensity
        self.bold = bold
        self.italic = italic
        self.underline = underline

    def sequence(self):
        sequence = []
        if self.name == "none":
            pass
        elif self.name == "reset":
            sequence.append("0")
        else:
            if self.foreground:
                foreground_parameter = 3
            else:
                foreground_parameter = 4
            if self.intensity:
                foreground_parameter += 6
            sequence.append("%d%d" % (foreground_parameter,
                                      self.names.index(self.name)))
        if self.bold:
            sequence.append("1")
        if self.italic:
            sequence.append("3")
        if self.underline:
            sequence.append("4")
        return sequence
    sequence = property(sequence)

    def escape_sequence(self):
        return "\033[%sm" % ";".join(self.sequence)
    escape_sequence = property(escape_sequence)

    def __add__(self, other):
        return MixColor([self, other])

class MixColor(object):
    def __init__(self, colors):
        self.colors = colors

    def sequence(self):
        return sum([color.sequence for color in self.colors], [])
    sequence = property(sequence)

    def escape_sequence(self):
        return "\033[%sm" % ";".join(self.sequence)
    escape_sequence = property(escape_sequence)

    def __add__(self, other):
        return MixColor([self, other])

COLORS = {
    "black": Color("black", bold=True),
    "black-back": Color("black", foreground=False),
    "red": Color("red", bold=True),
    "red-back": Color("red", foreground=False),
    "green": Color("green", bold=True),
    "green-back": Color("green", foreground=False),
    "yellow": Color("yellow", bold=True),
    "blue": Color("blue", bold=True),
    "magenta": Color("magenta", bold=True),
    "magenta-back": Color("magenta", foreground=False),
    "cyan": Color("cyan", bold=True),
    "cyan-back": Color("cyan", foreground=False),
    "white": Color("white", bold=True),
    "white-back": Color("white", foreground=False),
    "reset": Color("reset"),
    }

SCHEMES = {
    "default": {
        "success": COLORS["green"],
        "notification": COLORS["cyan"],
        "pending": COLORS["magenta"],
        "omission": COLORS["white-back"] + COLORS["blue"],
        "failure": COLORS["red"],
        "error": COLORS["yellow"],
        "test-case-name": COLORS["green-back"] + COLORS["white"],
        "file-name": COLORS["cyan"],
        "line-number": COLORS["yellow"],
        "content": None,
        }
    }



import os
import re
import shutil
import sys

try:
    from exceptions import *
except ImportError:
    pass

try:
    import syslog
except ImportError:
    pass

import pikzie
import pikzie.pretty_print as pp
import test.utils
try:
    import builtins
except ImportError:
    builtins = __builtins__

base_path = os.path.dirname(__file__)
tmp_path = os.path.join(base_path, "tmp")
nonexistent_path = os.path.join(base_path, "nonexistent")

class TestAssertions(pikzie.TestCase, test.utils.Assertions):
    """Tests for assertions."""

    class ComparableError(Exception):
        def __init__(self, message):
            self.message = message

        def __repr__(self):
            return "%s(%r,)" % (type(self).__name__, self.message)

        def __eq__(self, other):
            return isinstance(other, self.__class__) and \
                    self.message == other.message

    class TestCase(pikzie.TestCase):
        def setup(self):
            shutil.rmtree(tmp_path, True)
            shutil.rmtree(nonexistent_path, True)

        def test_fail(self):
            self.fail("Failed!!!")

        def test_pend(self):
            self.assert_equal(3, 1 + 2)
            self.pend("Pending!!!")
            self.assert_equal(5, 3 + 2)

        def test_notify(self):
            self.assert_equal(3, 1 + 2)
            self.notify("Notification!!!")
            self.assert_equal(5, 3 + 2)

        def test_assert_none(self):
            self.assert_none(None)
            self.assert_none(False)

        def test_assert_none_with_message(self):
            self.assert_none(True, "Message")

        def test_assert_not_none(self):
            self.assert_not_none(False)
            self.assert_not_none(None)

        def test_assert_true_for_none(self):
            self.assert_true(None)

        def test_assert_true_for_boolean(self):
            self.assert_true(True)
            self.assert_true(False)
            self.assert_true(True)

        def test_assert_true_for_integer(self):
            self.assert_true(10)
            self.assert_true(0)
            self.assert_true(-100)

        def test_assert_true_for_string(self):
            self.assert_true("String")
            self.assert_true("")
            self.assert_true("STRING")

        def test_assert_false_for_none(self):
            self.assert_false(None)

        def test_assert_false_for_boolean(self):
            self.assert_false(False)
            self.assert_false(True)
            self.assert_false(False)

        def test_assert_false_for_integer(self):
            self.assert_false(0)
            self.assert_false(10)
            self.assert_false(0)

        def test_assert_false_for_string(self):
            self.assert_false("")
            self.assert_false("STRING")
            self.assert_false("")

        def test_assert_equal(self):
            self.assert_equal(1, 1)
            self.assert_equal(2, 3)
            self.assert_equal(5, 5)

        def test_assert_not_equal(self):
            self.assert_not_equal(2, 3)
            self.assert_not_equal(2, 2)
            self.assert_not_equal(5, 1)

        def test_assert_not_equal_different_repr(self):
            class AlwaysEqual(object):
                def __init__(self, repr):
                    self.repr = repr

                def __ne__(self, other):
                    return False

                def __repr__(self):
                    return repr(self.repr)

            self.assert_not_equal(AlwaysEqual("abc"), AlwaysEqual("aBc"))

        def test_assert_in_delta(self):
            self.assert_in_delta(0.5, 0.5001, 0.01)
            self.assert_in_delta(0.5, 0.5001, 0.001)
            self.assert_in_delta(0.5, 0.5001, 0.0001)
            self.assert_in_delta(0.5, 0.5001, 0.00001)
            self.assert_in_delta(0.5, 0.5001, 0.000001)

        def test_assert_match(self):
            self.assert_match("abc", "abcde")
            self.assert_match("abc", "Xabcde")

        def test_assert_match_re(self):
            self.assert_match(re.compile("xyz"), "xyzabc")
            self.assert_match(re.compile("xyz"), "abcxyz")

        def test_assert_not_match(self):
            self.assert_not_match("abc", "Xabcde")
            self.assert_not_match("abc", "abcde")

        def test_assert_not_match_re(self):
            self.assert_not_match(re.compile("xyz", re.I), "abcXYZ")
            self.assert_not_match(re.compile("xyz", re.I), "XYZabc")

        def test_assert_search(self):
            self.assert_search("bcd", "abcde")
            self.assert_search("bcd", "bcd")
            self.assert_search("bcd", "abCde")

        def test_assert_search_re(self):
            self.assert_search(re.compile("xyz"), "AxyzA")
            self.assert_search(re.compile("xyz"), "xyz")
            self.assert_search(re.compile("xyz"), "AxYzA")

        def test_assert_not_found(self):
            self.assert_not_found("bcd", "abCde")
            self.assert_not_found(re.compile("bcd"), "abCde")
            self.assert_not_found(re.compile("bcd", re.I | re.L), "abCde")

        def test_assert_hasattr(self):
            self.assert_hasattr("string", "ljust")
            self.assert_hasattr("string", "strip")
            self.assert_hasattr("string", "Strip")
            self.assert_hasattr("string", "index")

        def test_assert_callable(self):
            class Callable(object):
                def __call__(self):
                    return "X"

            self.assert_callable(Callable())
            self.assert_callable(lambda : "zzz")
            self.assert_callable("string")
            self.assert_callable(self.test_assert_callable)

        def test_assert_raise_call(self):
            def raise_name_error():
                unknown_name
            exception = self.assert_raise_call(NameError, raise_name_error)
            try:
                unknown_name
            except NameError as name_error:
                expected_message = str(name_error)
            self.assert_equal(expected_message, str(exception))

            def nothing_raised():
                "nothing raised"
            self.assert_raise_call(NameError, nothing_raised)

        def test_assert_raise_call_different_error(self):
            def raise_zero_division_error():
                1 / 0
            self.assert_raise_call(NameError, raise_zero_division_error)

        def test_assert_raise_call_instance(self):
            def raise_error():
                raise TestAssertions.ComparableError("raise error")
            self.assert_raise_call(TestAssertions.ComparableError("raise error"),
                                   raise_error)
            self.assert_raise_call(TestAssertions.ComparableError("not error"),
                                   raise_error)

        def test_assert_nothing_raised_call(self):
            self.assert_equal(123, self.assert_nothing_raised_call(lambda : 123))
            def raise_zero_division_error():
                1 / 0
            self.assert_nothing_raised_call(raise_zero_division_error)

        def test_assert_run_command(self):
            process = self.assert_run_command(["echo", "12345"])
            self.assert_equal(b"12345\n", process.stdout.read())
            self.assert_run_command("false")

        def test_assert_run_command_unknown(self):
            self.assert_run_command(["unknown", "arg1", "arg2"])

        def test_assert_search_syslog_call(self):
            self.assert_search_syslog_call("find me!+",
                                           syslog.syslog, "find me!!!")
            self.assert_search_syslog_call("fix me!",
                                           syslog.syslog, "FIXME!!!")

        def test_assert_exists(self):
            self.assert_exists(__file__)
            self.assert_exists(nonexistent_path)

        def test_assert_not_exists(self):
            self.assert_not_exists(nonexistent_path)
            self.assert_not_exists(__file__)

        def test_assert_open_file(self):
            file = self.assert_open_file(__file__)
            self.assert_equal("r", file.mode)
            file = self.assert_open_file(tmp_path, "w")
            self.assert_equal("w", file.mode)
            self.assert_open_file(nonexistent_path)

        def test_assert_try_call(self):
            self.n = 0
            def succeed_on_5th_try():
                self.n += 1
                self.assert_equal(5, self.n)
            self.assert_try_call(1, 0.01, succeed_on_5th_try)

            def never_succeed():
                self.fail("Never succeed")
            self.assert_try_call(0.1, 0.01, never_succeed)

        def test_assert_kernel_symbol(self):
            address = self.assert_kernel_symbol("printk")
            self.assert_not_none(address)
            self.assert_kernel_symbol("nonexistent")

    def test_fail(self):
        """Test for fail"""
        self.assert_result(False, 1, 0, 1, 0, 0, 0, 0,
                           [("F", "TestCase.test_fail", "Failed!!!", None)],
                           ["test_fail"])

    def test_pend(self):
        self.assert_result(True, 1, 1, 0, 0, 1, 0, 0,
                           [('P',
                             "TestCase.test_pend",
                             "Pending!!!",
                             None)],
                           ["test_pend"])

    def test_notify(self):
        self.assert_result(True, 1, 2, 0, 0, 0, 0, 1,
                           [('N',
                             "TestCase.test_notify",
                             "Notification!!!",
                             None)],
                           ["test_notify"])

    def test_assert_none(self):
        self.assert_result(False, 2, 1, 2, 0, 0, 0, 0,
                           [("F",
                             "TestCase.test_assert_none",
                             "expected: <False> is None",
                             None),
                            ("F",
                             "TestCase.test_assert_none_with_message",
                             "Message\n"
                             "expected: <True> is None",
                             None)],
                           ["test_assert_none",
                            "test_assert_none_with_message"])

    def test_assert_not_none(self):
        self.assert_result(False, 1, 1, 1, 0, 0, 0, 0,
                           [("F",
                             "TestCase.test_assert_not_none",
                             "expected: not None",
                             None)],
                           ["test_assert_not_none"])

    def test_assert_true(self):
        self.assert_result(False, 4, 3, 4, 0, 0, 0, 0,
                           [("F",
                             "TestCase.test_assert_true_for_none",
                             "expected: <None> is a true value",
                             None),
                            ("F",
                             "TestCase.test_assert_true_for_boolean",
                             "expected: <False> is a true value",
                             None),
                            ("F",
                             "TestCase.test_assert_true_for_integer",
                             "expected: <0> is a true value",
                             None),
                            ("F",
                             "TestCase.test_assert_true_for_string",
                             "expected: <''> is a true value",
                             None)],
                           ["test_assert_true_for_none",
                            "test_assert_true_for_boolean",
                            "test_assert_true_for_integer",
                            "test_assert_true_for_string"])

    def test_assert_false(self):
        self.assert_result(False, 4, 4, 3, 0, 0, 0, 0,
                           [("F",
                             "TestCase.test_assert_false_for_boolean",
                             "expected: <True> is a false value",
                             None),
                            ("F",
                             "TestCase.test_assert_false_for_integer",
                             "expected: <10> is a false value",
                             None),
                            ("F",
                             "TestCase.test_assert_false_for_string",
                             "expected: <'STRING'> is a false value",
                             None)],
                           ["test_assert_false_for_none",
                            "test_assert_false_for_boolean",
                            "test_assert_false_for_integer",
                            "test_assert_false_for_string"])

    def test_assert_equal(self):
        self.assert_result(False, 1, 1, 1, 0, 0, 0, 0,
                           [('F',
                             'TestCase.test_assert_equal',
                             "expected: <2>\n"
                             " but was: <3>",
                             None)],
                           ["test_assert_equal"])

    def test_assert_not_equal(self):
        self.assert_result(False, 2, 1, 2, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_not_equal",
                             "not expected: <2>\n"
                             "     but was: <2>",
                             None),
                            ("F",
                             "TestCase.test_assert_not_equal_different_repr",
                             "not expected: <'abc'>\n"
                             "     but was: <'aBc'>\n"
                             "\n"
                             "diff:\n"
                             "- 'abc'\n"
                             "?   ^\n"
                             "+ 'aBc'\n"
                             "?   ^",
                             None)],
                           ["test_assert_not_equal",
                            "test_assert_not_equal_different_repr"])

    def test_assert_in_delta(self):
        expected = 0.5
        actual = 0.5001
        delta = 0.00001
        params = (expected, delta, [expected - delta, expected + delta], actual)
        self.assert_result(False, 1, 3, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_in_delta",
                             "expected: <%r+-%r %r>\n"
                             " but was: <%r>" % params,
                             None)],
                           ["test_assert_in_delta"])

    def test_assert_match(self):
        pattern = re.compile('xyz')
        self.assert_result(False, 2, 2, 2, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_match",
                             "expected: re.match('abc', 'Xabcde') "
                               "doesn't return None\n"
                             " pattern: </abc/>\n"
                             "  target: <'Xabcde'>",
                             None),
                            ('F',
                             "TestCase.test_assert_match_re",
                             "expected: re.match(%s, 'abcxyz') "
                               "doesn't return None\n"
                             " pattern: <%s>\n"
                             "  target: <'abcxyz'>" % \
                                 (pp.format_re_repr(pattern),
                                  pp.format_re(pattern)),
                             None)],
                           ["test_assert_match",
                            "test_assert_match_re"])

    def test_assert_not_match(self):
        pattern = re.compile('xyz', re.IGNORECASE)
        self.assert_result(False, 2, 2, 2, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_not_match",
                             "expected: re.match('abc', 'abcde') returns None\n"
                             " pattern: </abc/>\n"
                             "  target: <'abcde'>",
                             None),
                            ('F',
                             "TestCase.test_assert_not_match_re",
                             "expected: re.match(%s, 'XYZabc') returns None\n"
                             " pattern: <%s>\n"
                             "  target: <'XYZabc'>" % \
                                 (pp.format_re_repr(pattern),
                                  pp.format_re(pattern)),
                             None)],
                           ["test_assert_not_match",
                            "test_assert_not_match_re"])

    def test_assert_search(self):
        pattern = re.compile('xyz')
        self.assert_result(False, 2, 4, 2, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_search",
                             "expected: re.search('bcd', 'abCde') "
                               "doesn't return None\n"
                             " pattern: </bcd/>\n"
                             "  target: <'abCde'>",
                             None),
                            ('F',
                             "TestCase.test_assert_search_re",
                             "expected: re.search(%s, 'AxYzA') "
                               "doesn't return None\n"
                             " pattern: <%s>\n"
                             "  target: <'AxYzA'>" % \
                                 (pp.format_re_repr(pattern),
                                  pp.format_re(pattern)),
                             None)],
                           ["test_assert_search",
                            "test_assert_search_re"])

    def test_assert_not_found(self):
        pattern = re.compile('bcd', re.IGNORECASE | re.LOCALE)
        self.assert_result(False, 1, 2, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_not_found",
                             "expected: re.search(%s, 'abCde') returns None\n"
                             " pattern: <%s>\n"
                             "  target: <'abCde'>" % \
                                 (pp.format_re_repr(pattern),
                                  pp.format_re(pattern)),
                             None)],
                           ["test_assert_not_found"])

    def test_assert_hasattr(self):
        self.assert_result(False, 1, 2, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_hasattr",
                             "expected: hasattr('string', 'Strip')",
                             None)],
                           ["test_assert_hasattr"])

    def test_assert_callable(self):
        self.assert_result(False, 1, 2, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_callable",
                             "expected: callable('string')",
                             None)],
                           ["test_assert_callable"])

    def test_assert_raise_call(self):
        zero_division_error = None
        try:
            1 / 0
        except ZeroDivisionError as exception:
            zero_division_error = exception
        self.assert_result(False, 3, 8, 3, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_raise_call",
                             "expected: <%s> is raised\n"
                             " but was: %s() nothing raised" %
                             (NameError,
                              "test_assertions.nothing_raised"),
                             None),
                            ('F',
                             "TestCase.test_assert_raise_call_different_error",
                             "expected: <%s> is raised\n"
                             " but was: <%s>(%s)" %
                             (NameError,
                              type(zero_division_error),
                              zero_division_error),
                             None),
                            ('F',
                             "TestCase.test_assert_raise_call_instance",
                             "expected: <%r>\n"
                             " but was: <%r>" %
                             (self.ComparableError("not error"),
                              self.ComparableError("raise error")),
                             None)],
                           ["test_assert_raise_call",
                            "test_assert_raise_call_different_error",
                            "test_assert_raise_call_instance"])

    def test_assert_nothing_raised_call(self):
        zero_division_error = None
        try:
            1 / 0
        except ZeroDivisionError as exception:
            zero_division_error = exception
        self.assert_result(False, 1, 4, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_nothing_raised_call",
                             "expected: %s() nothing raised\n"
                             " but was: <%s>(%s) is raised" % \
                                 ("test_assertions."
                                  "raise_zero_division_error",
                                  type(zero_division_error),
                                  zero_division_error),
                             None)],
                           ["test_assert_nothing_raised_call"])

    def test_assert_run_command(self):
        try:
            import subprocess
        except ImportError:
            self.pend("subprocess module isn't available.")
        os_error = None
        try:
            subprocess.Popen(["unknown", "arg1", "arg2"])
        except OSError as exception:
            os_error = exception
        self.assert_result(False, 2, 2, 2, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_run_command",
                             "expected: <%s> is successfully finished\n"
                             " but was: <%d> is returned as exit code" % \
                                 ("'false'", 1),
                             None),
                            ('F',
                             "TestCase.test_assert_run_command_unknown",
                             "expected: <%s> is successfully ran\n"
                             " but was: <%s>(%s) is raised and failed to ran" \
                                 % ("['unknown', 'arg1', 'arg2']",
                                    type(os_error),
                                    os_error),
                             None)],
                           ["test_assert_run_command",
                            "test_assert_run_command_unknown"])

    def test_assert_search_syslog_call(self):
        if not hasattr(sys.modules[__name__], "syslog"):
            self.omit("syslog isn't supported on this environment")

        if not os.access("/var/log/messages", os.R_OK):
            self.pend("can't read /var/log/messages.")
        detail = \
            "expected: <%s> is found in <%s>\n" \
            " content: <b?'.*FIXME!!!.*'>" % \
            ("/fix me!/u?", "'/var/log/messages'")
        self.assert_result(False, 1, 2, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_search_syslog_call",
                             re.compile(detail),
                             None)],
                           ["test_assert_search_syslog_call"])

    def test_assert_exists(self):
        self.assert_result(False, 1, 0, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_exists",
                             "expected: <%s> exists" % nonexistent_path,
                             None)],
                           ["test_assert_exists"])

    def test_assert_not_exists(self):
        self.assert_result(False, 1, 0, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_not_exists",
                             "expected: <%s> doesn't exists" % __file__,
                             None)],
                           ["test_assert_not_exists"])

    def test_assert_open_file(self):
        file_not_found_error = getattr(builtins, "FileNotFoundError", IOError)
        self.assert_result(False, 1, 2, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_open_file",
                             "expected: open('%s') succeeds\n"
                             " but was: <%s>(%s) is raised" % \
                                 (nonexistent_path,
                                  file_not_found_error,
                                  "[Errno 2] No such file or directory: '%s'" % \
                                      nonexistent_path),
                             None)],
                           ["test_assert_open_file"])

    def test_try_call(self):
        self.assert_result(False, 1, 1, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_try_call",
                             "expected: %s succeeds\n"
                             " timeout: <0.1> seconds\n"
                             "interval: <0.01> seconds\n"
                             " but was:\n"
                             "Never succeed" % \
                                 ("test_assertions.never_succeed()",),
                             None)],
                           ["test_assert_try_call"])

    def test_kernel_symbol(self):
        if not hasattr(os, "uname"):
            self.omit("only for Linux environment")
        if os.uname()[0] != "Linux":
            self.omit("only for Linux environment")
        if not os.path.exists("/proc/kallsyms"):
            self.omit("require /proc/kallsyms")
        self.assert_result(False, 1, 1, 1, 0, 0, 0, 0,
                           [('F',
                             "TestCase.test_assert_kernel_symbol",
                             "expected: <'nonexistent'> is in kernel symbols",
                             None)],
                           ["test_assert_kernel_symbol"])



import pikzie
import test.utils

class TestPriority(pikzie.TestCase, test.utils.Assertions):
    """Tests for priority mode."""

    class TestCase(pikzie.TestCase):
        def test_must(self):
            pass
        test_must = pikzie.priority("must")(test_must)

        def test_important(self):
            pass
        test_important = pikzie.priority("important")(test_important)

        def test_high(self):
            pass
        test_high = pikzie.priority("high")(test_high)

        def test_normal(self):
            pass
        test_normal = pikzie.priority("normal")(test_normal)

        def test_low(self):
            pass
        test_low = pikzie.priority("low")(test_low)

        def test_never(self):
            pass
        test_never = pikzie.priority("never")(test_never)

    def test_priority(self):
        self.assert_need_to_run_according_to_priority("test_must", 1.0, 0.0001)
        self.assert_need_to_run_according_to_priority("test_important",
                                                      0.9, 0.09)
        self.assert_need_to_run_according_to_priority("test_high", 0.70, 0.1)
        self.assert_need_to_run_according_to_priority("test_normal", 0.5, 0.1)
        self.assert_need_to_run_according_to_priority("test_low", 0.25, 0.1)
        self.assert_need_to_run_according_to_priority("test_never", 0.0, 0.0001)

    def assert_need_to_run_according_to_priority(self, test_name,
                                                 expected, delta):
        n = 1000
        n_need_to_run = 0
        for i in range(0, n):
            if self.TestCase(test_name)._need_to_run_according_to_priority():
                n_need_to_run +=1
        self.assert_in_delta(expected, float(n_need_to_run) / n, delta)



# Copyright (C) 2010  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil

__all__ = ["mkdir_p", "rm_rf", "cp_a"]

def mkdir_p(path, *arguments, **parameters):
    os.makedirs(path, *arguments, **parameters)

def rm_rf(path):
    if not os.path.exists(path):
        return
    if os.path.isdir(path):
        shutil.rmtree(path, ignore_errors=True)
    else:
        os.remove(path)

def cp_a(src, dest):
    shutil.copytree(src, dest)



# Copyright (C) 2009-2015  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pikzie.color import *

def format_metadata(metadata, need_newline=False):
    if metadata is None:
        return ""
    format_keys = filter(lambda key: key != "data", metadata)
    formatted_metadata = ["  %s: %s" % (key, metadata[key])
                          for key
                          in sorted(format_keys)]
    if len(formatted_metadata) == 0:
        return ""
    result = "\n".join(formatted_metadata)
    if need_newline:
        result += "\n"
    return result

class TestResult(object):
    pass

class Success(TestResult):
    name = "success"

    def __init__(self, test):
        self.fault = False
        self.critical = False
        self.symbol = "."
        self.test = test
        self.traceback = None

    def detail(self):
        return ""

class Notification(TestResult):
    name = "notification"

    def __init__(self, test, message, traceback):
        self.fault = True
        self.critical = False
        self.symbol = "N"
        self.test = test
        self.message = message
        self.traceback = traceback

    def title(self):
        return "Notification: %s: %s" % (self.test, self.message)

    def detail(self):
        return ""

class Omission(TestResult):
    name = "omission"

    def __init__(self, test, message, traceback):
        self.fault = True
        self.critical = False
        self.symbol = "O"
        self.test = test
        self.message = message
        self.traceback = traceback

    def title(self):
        return "Omission: %s: %s" % (self.test, self.message)

    def detail(self):
        return ""

class Pending(TestResult):
    name = "pending"

    def __init__(self, test, message, traceback):
        self.fault = True
        self.critical = False
        self.symbol = "P"
        self.test = test
        self.message = message
        self.traceback = traceback

    def title(self):
        return "Pending: %s: %s" % (self.test, self.message)

    def detail(self):
        return ""

class Failure(TestResult):
    name = "failure"

    def __init__(self, test, message, traceback, expected=None, actual=None):
        self.fault = True
        self.critical = True
        self.symbol = "F"
        self.test = test
        self.message = message
        self.traceback = traceback
        self.expected = expected
        self.actual = actual

    def title(self):
        if len(self.traceback) == 0:
            return "Failure: %s" % self.test
        else:
            return "Failure: %s: %s" % (self.test, self.traceback[0].content)

    def detail(self):
        return self.message

class Error(TestResult):
    name = "error"

    def __init__(self, test, exception_type, message, traceback):
        self.fault = True
        self.critical = True
        self.symbol = "E"
        self.test = test
        self.exception_type = exception_type
        self.message = message
        self.traceback = traceback

    def title(self):
        return "Error: %s" % self.test

    def detail(self):
        return "%s: %s" % (self.exception_type, self.message)

FAULT_ORDER = [Notification, Omission, Pending, Failure, Error]

def fault_compare_key(fault):
    return FAULT_ORDER.index(type(fault))



#!/usr/bin/env python
#
# Copyright (C) 2009-2015  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import re
import glob
import subprocess
import pydoc
import shutil
import gettext
try:
    import setuptools
except ImportError:
    pass

from distutils.core import setup
from distutils.cmd import Command
from distutils.dist import DistributionMetadata

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "lib"))

import pikzie

package_name = "Pikzie"
distribution_name = package_name.lower()
version = pikzie.version
sf_project_name = "Pikzie"
sf_project_id = package_name.lower()
sf_package_name = package_name.lower()
sf_user = "ktou"
sf_host = "%s@web.sourceforge.net" % sf_user
sf_htdocs = "/home/groups/p/pi/pikzie/htdocs"

long_description = re.split("\n.+\n=+", open("README.rst").read())[5].strip()
description = re.sub("\n", " ", long_description.split("\n\n")[0])

def get_fullname(self):
    return "%s-%s" % (distribution_name, self.get_version())
DistributionMetadata.get_fullname = get_fullname

def _run(*command):
    return_code = _run_without_check(*command)
    if return_code != 0:
        raise RuntimeError("failed to run <%d>: %s" % \
                               (return_code, " ".join(command)))

def _run_without_check(*command):
    print(" ".join(command))
    return subprocess.call(command)

class update_po(Command):
    description = "update *.po"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import textwrap
        command = ["pygettext", "--extract-all", "--default-domain", "pikzie",
                   "--docstrings", "--output-dir", "po"]
        command.extend(glob.glob("lib/*.py"))
        command.extend(glob.glob("lib/*/*.py"))
        command.extend(glob.glob("lib/*/*/*.py"))
        _run(*command)

        pot = file("po/pikzie.pot", "r").read()
        docstring_msgid_re = re.compile("^#, docstring\nmsgid(.+?)^msgstr",
                                        re.M | re.DOTALL)
        def strip_spaces(match_object):
            docstring = match_object.group(1).strip()
            docstring = re.compile("^\"|\"$", re.M).sub("", docstring)
            docstring = re.sub("\n", "", docstring)
            docstring = re.sub(r"(?<!\\)\\n", "\n", docstring)
            docstring = textwrap.dedent(docstring).strip()
            docstring = re.compile("^(.*?)$", re.M).sub(r'"\1\\n"', docstring)
            docstring = re.sub("\\\\n\"$", "\"", docstring)
            return "#, docstring\nmsgid \"\"\n%s\nmsgstr" % docstring
        pot = docstring_msgid_re.sub(strip_spaces, pot)
        pot_file = file("po/pikzie.pot", "w")
        pot_file.write(pot)
        pot_file.close()

        for po in glob.glob("po/*.po"):
            _run("msgmerge", "--update", po, "po/pikzie.pot")

class update_mo(Command):
    description = "update *.mo"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        for po in glob.glob("po/*.po"):
            (lang, ext) = os.path.splitext(os.path.basename(po))
            mo_dir = os.path.join("data", "locale", lang, "LC_MESSAGES")
            if not os.access(mo_dir, os.F_OK):
                os.makedirs(mo_dir)
            _run("msgfmt", "--output-file", os.path.join(mo_dir, "pikzie.mo"),
                 po)

class update_doc(Command):
    description = "update documentation"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        self._generate_assertions_html(None)
        self._generate_assertions_html("ja")
        _run("rst2html", "README.rst", "html/readme.html")
        _run("rst2html", "README.ja.rst", "html/readme.html.ja")
        _run("rst2html", "NEWS.rst", "html/news.html")
        _run("rst2html", "NEWS.ja.rst", "html/news.html.ja")

    def _generate_assertions_html(self, lang):
        object = pikzie.assertions.Assertions
        html_name = "html/assertions.html"
        translation = None
        if lang:
            html_name = "%s.%s" % (html_name, lang)
            translation = gettext.translation("pikzie", "data/locale", [lang])

        print(html_name)

        original_getdoc = pydoc.getdoc
        def getdoc(object):
            document = original_getdoc(object)
            if document == "":
                return document
            else:
                return translation.gettext(document)
        if translation:
            pydoc.getdoc = getdoc
        page = pydoc.html.page(pydoc.describe(object),
                               pydoc.html.document(object, "assertions"))
        pydoc.getdoc = original_getdoc

        html = file(html_name, "w")
        html.write(page.strip())
        html.close()

class upload_doc(Command):
    description = "upload documentation"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        sdist = self.reinitialize_command("update_doc")
        self.run_command("update_doc")
        html_files = glob.glob("html/*.html*")
        for html in html_files:
            if html.startswith("html/index.html"):
                continue
            if html.endswith(".ja"):
                footer = file("html/footer.ja").read()
            else:
                footer = file("html/footer").read()
            self._prepare_html(html, footer)
        commands = ["scp"]
        commands.extend(html_files)
        commands.append("html/pikzie.css")
        commands.append("%s:%s/" % (sf_host, sf_htdocs))
        _run(*commands)

    def _prepare_html(self, html, footer):
        html_file = file(html, "rw+")
        content = html_file.read()
        content = re.sub("</head>",
                         r'<link rel="stylesheet" href="pikzie.css" type="text/css" />\n' +
                         r'</head>',
                         content)
        content = re.sub("<body([^>]*?)>",
                         r'<body\1><div class="main">\n',
                         content)
        content = re.sub("</body>",
                         r'</div>\n%s\n</body>' % footer,
                         content)
        html_file.seek(0)
        html_file.write(content)
        html_file.close

class release(Command):
    description = "release package to SF.net"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        sdist = self.reinitialize_command("sdist")
        self.run_command("sdist")
        archive_name = "%s.tar.gz" % self.distribution.get_fullname()
        os.rename("dist/%s" % archive_name, archive_name)
        _run("misc/release.rb", sf_user,
             sf_project_id, sf_project_name, sf_package_name,
             version, "README.rst", "NEWS.rst",
             archive_name)

class tag(Command):
    description = "tag %s" % version
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        _run("git", "tag", version, "-a", "-m", "released %s!!!" % version)

download_url = "http://downloads.sourceforge.net/project/pikzie/pikzie/%s/pikzie-%s.tar.gz" % (version, version)
setup(name=package_name,
      version=version,
      description=description,
      long_description=long_description,
      author="Kouhei Sutou",
      author_email="kou@clear-code.com",
      url="http://pikzie.sourceforge.net/",
      download_url=download_url,
      license="LGPL",
      package_dir={'': 'lib'},
      packages=["pikzie", "pikzie.ui"],
      classifiers=[
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Natural Language :: Japanese",
        "Natural Language :: English",
        ],
      cmdclass={"update_po": update_po,
                "update_mo": update_mo,
                "update_doc": update_doc,
                "upload_doc": upload_doc,
                "release": release,
                "tag": tag})



# Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import math
import re

from optparse import OptionValueError

import pikzie.color
from pikzie.core import *
from pikzie.results import *
import pikzie.pretty_print as pp

VERBOSE_LEVEL_SILENT = 0
VERBOSE_LEVEL_NORMAL = 1
VERBOSE_LEVEL_VERBOSE = 2

class ConsoleTestRunner(object):
    def setup_color_option(cls, group):
        available_values = "[yes|true|no|false|auto]"
        def store_use_color(option, opt, value, parser):
            if value == "yes" or value == "true":
                parser.values.verbose_level = True
            elif value == "no" or value == "false":
                parser.values.use_color = False
            elif value == "auto" or value is None:
                pass
            else:
                format = "should be one of %s: %s"
                raise OptionValueError(format % (available_values, value))

        help = "Output log with colors %s (default: auto)" % available_values
        group.add_option("-c", "--color",
                         action="callback", callback=store_use_color,
                         dest="use_color", nargs=1, type="string",
                         metavar="MODE", help=help)
    setup_color_option = classmethod(setup_color_option)

    def setup_color_scheme_option(cls, group):
        available_schemes = list(pikzie.color.SCHEMES.keys())
        help = "Use color scheme named SCHEME [%s]" % "|".join(available_schemes)
        group.add_option("--color-scheme", dest="color_scheme",
                         choices=available_schemes,
                         metavar="SCHEME", help=help)
    setup_color_scheme_option = classmethod(setup_color_scheme_option)

    def setup_verbose_option(cls, group):
        available_values = "[s|silent|n|normal|v|verbose]"
        def store_verbose_level(option, opt, value, parser):
            if value == "s" or value == "silent":
                parser.values.verbose_level = VERBOSE_LEVEL_SILENT
            elif value == "n" or value == "normal":
                parser.values.verbose_level = VERBOSE_LEVEL_NORMAL
            elif value == "v" or value == "verbose":
                parser.values.verbose_level = VERBOSE_LEVEL_VERBOSE
            else:
                format = "should be one of %s: %s"
                raise OptionValueError(format % (available_values, value))

        help = "Output level %s (default: normal)" % available_values
        group.add_option("-v", "--verbose",
                         action="callback", callback=store_verbose_level,
                         dest="verbose_level", nargs=1, type="string", help=help)
    setup_verbose_option = classmethod(setup_verbose_option)

    def setup_options(cls, parser):
        group = parser.add_option_group("Console UI", "Options for console UI")
        cls.setup_color_option(group)
        cls.setup_color_scheme_option(group)
        cls.setup_verbose_option(group)
    setup_options = classmethod(setup_options)

    def __init__(self, output=sys.stdout, use_color=None, verbose_level=None,
                 color_scheme=None):
        if use_color is None:
            use_color = self._detect_color_availability()
        self.use_color = use_color
        if verbose_level is None:
            verbose_level = VERBOSE_LEVEL_NORMAL
        self.verbose_level = verbose_level
        self.output = output
        self.color_scheme = pikzie.color.SCHEMES[color_scheme or "default"]
        self.reset_color = pikzie.color.COLORS["reset"]

    def run(self, test, listeners=[]):
        "Run the given test case or test suite."
        context = TestRunnerContext()
        context.add_listener(self)
        context.add_listeners(listeners)
        test.run(context)
        return context

    def on_start_test_case(self, context, test_case):
        description = self._generate_test_case_description(test_case)
        self._writeln("%s:%s" % (test_case.__name__, description),
                      self.color_scheme["test-case-name"],
                      level=VERBOSE_LEVEL_VERBOSE)

    def on_finish_test_suite(self, context, test_suite):
        if self.verbose_level == VERBOSE_LEVEL_NORMAL:
            self._writeln()
        self._print_faults(context)
        self._writeln("Finished in %.3f seconds" % context.elapsed)
        self._writeln()
        self._writeln(context.summary(), self._result_color(context))

    def _generate_test_case_description(self, test_case):
        if not test_case.__doc__:
            return ""

        if re.search("\n", test_case.__doc__):
            lines = test_case.__doc__.rstrip().split("\n")
            min_indent = 1000
            for line in lines[1:]:
                if re.match(" *$", line):
                    continue
                min_indent = min([min_indent, re.match(" *", line).end()])
            lines = lines[0:1] + map(lambda line: line[min_indent:],
                                     lines[1:])
            lines = map(lambda line: " " + line, lines)
            return "\n%s" % "\n".join(lines)
        else:
            return " %s" % test_case.__doc__

    def on_start_test(self, context, test):
        self._n_notifications = 0

        if test.description():
            self._writeln(" %s" % test.description(),
                          level=VERBOSE_LEVEL_VERBOSE)

        test_name = test.short_name()
        indent = "  "
        context_mark = "."
        spaces = len(indent) + len(":") + len(context_mark)
        tab = "\t" * int((8 * 9 + spaces - len(test_name)) / 8)
        self._write("%s%s:%s" % (indent, test_name, tab),
                    level=VERBOSE_LEVEL_VERBOSE)

    def on_success(self, context, test):
        self._flood_notifications()
        self._write(".", self.color_scheme["success"])

    def _on_fault(self, context, fault):
        self._flood_notifications()
        self._write_fault(fault)

    on_failure = _on_fault
    on_error = _on_fault
    on_pending = _on_fault
    on_omission = _on_fault

    def on_notification(self, context, notification):
        self._pool_notification(notification)
        if self.verbose_level != VERBOSE_LEVEL_VERBOSE:
            self._write_fault(notification)
            return

        if self._n_notifications == 1:
            self._write_fault(notification)

    def on_finish_test(self, context, test):
        self._flood_notifications()
        self._writeln(level=VERBOSE_LEVEL_VERBOSE)

    def on_finish_test_case(self, context, test_case):
        self._writeln(level=VERBOSE_LEVEL_VERBOSE)

    def _pool_notification(self, notification):
        self._n_notifications += 1
        self._last_notification = notification

    def _flood_notifications(self):
        if self._n_notifications > 1:
            fault = self._last_notification
            color = self._fault_color(fault)
            if self._n_notifications > 3:
                message = "%d%s" % (self._n_notifications, fault.symbol)
            else:
                n_characters = self._n_notifications - 1
                message = fault.symbol * n_characters
            self._write(message, color, VERBOSE_LEVEL_VERBOSE)
        self._n_notifications = 0
        self._last_notification = None

    def _fault_color(self, fault):
        return self.color_scheme[fault.__class__.name]

    def _write(self, arg, color=None, level=VERBOSE_LEVEL_NORMAL):
        if self.verbose_level < level:
            return
        if self.use_color and color:
            arg = "%s%s%s" % (color.escape_sequence,
                              arg,
                              self.reset_color.escape_sequence)
        self.output.write(arg)
        self.output.flush()

    def _write_fault(self, fault, level=VERBOSE_LEVEL_NORMAL):
        self._write(fault.symbol, self._fault_color(fault), level)

    def _writeln(self, arg=None, color=None, level=VERBOSE_LEVEL_NORMAL):
        if arg:
            self._write(arg, color, level)
        self._write("\n", level=level)

    def _print_faults(self, context):
        size = len(context.faults)
        if size == 0:
            return
        self._writeln()
        index_format = "%%%dd) " % (math.floor(math.log10(size)) + 1)
        for i, fault in enumerate(context.faults):
            self._write(index_format % (i + 1))
            self._writeln(fault.title(), self._fault_color(fault))
            if fault.test._data_label():
                self._writeln("  data: %s" % (fault.test._data(),))
            metadata = format_metadata(fault.test.metadata)
            if metadata:
                self._writeln(metadata, self._fault_color(fault))
            self._print_traceback(fault.traceback)
            self._print_fault_message(fault)
            self._writeln()

    def _print_traceback(self, traceback):
        if len(traceback) == 0:
            return
        for entry in traceback:
            self._write(entry.file_name, self._file_name_color())
            self._write(":")
            self._write("%d" % entry.line_number, self._line_number_color())
            if entry.content:
                self._write(": ")
                self._write(entry.content, self._content_color())
            self._writeln()

    def _print_fault_message(self, fault):
        detail = fault.detail()
        if detail:
            self._writeln(str(detail))
        if hasattr(fault, "expected") and fault.expected and fault.actual:
            self._write("expected: <")
            self._write(fault.expected, self.color_scheme["success"])
            self._writeln(">")
            self._write(" but was: <")
            self._write(fault.actual, self._fault_color(fault))
            self._writeln(">")
            formatted_expected = pp.format_for_diff(fault.expected)
            formatted_actual = pp.format_for_diff(fault.actual)
            diff = pp.format_diff(formatted_expected, formatted_actual)
            if pp.is_interested_diff(diff):
                self._writeln("diff:")
                self._writeln(diff)
            if pp.need_fold(diff):
                folded_diff = pp.format_folded_diff(formatted_expected,
                                                    formatted_actual)
                self._writeln()
                self._writeln("folded diff:")
                self._writeln(folded_diff)

    def _result_color(self, context):
        if len(context.faults) == 0:
            return self.color_scheme["success"]
        else:
            sorted_faults = sorted(context.faults, key=fault_compare_key)
            return self._fault_color(sorted_faults[0])

    def _file_name_color(self):
        return self.color_scheme["file-name"]

    def _line_number_color(self):
        return self.color_scheme["line-number"]

    def _content_color(self):
        return self.color_scheme["content"]

    def _detect_color_availability(self):
        term = os.getenv("TERM")
        if term and (term.endswith("term") or
                     term.endswith("term-color") or
                     term.endswith("term-256color") or
                     term == "screen"):
            return True
        emacs = os.getenv("EMACS")
        if emacs and (emacs == "t"):
            return True
        return False



import pikzie
import test.utils

class TestMetadata(pikzie.TestCase, test.utils.Assertions):
    """Tests for test metadata."""

    class TestCase(pikzie.TestCase):
        def test_for_bug_111(self):
            self.fail("This is a bug!!!")
        test_for_bug_111 = pikzie.bug(111)(test_for_bug_111)

        def test_without_metadata(self):
            self.assert_equal(1, 1)

        def test_with_metadata(self):
            self.assert_equal(1, 1)
        test_with_metadata = pikzie.metadata("key", "value")(test_with_metadata)


    def test_bug(self):
        """Test for bug 111"""
        self.assert_metadata("bug", 111, "test_for_bug_111")

    def test_bug_result(self):
        self.assert_result(False, 1, 0, 1, 0, 0, 0, 0,
                           [("F",
                             "TestCase.test_for_bug_111",
                             "This is a bug!!!",
                             {"bug": 111})],
                           ["test_for_bug_111"])

    def test_unknown_metadata_for_test_without_metadata(self):
        self.assert_metadata("unknown key", None, "test_without_metadata")

    def test_unknown_metadata_for_test_with_metadata(self):
        self.assert_metadata("key", "value", "test_with_metadata")
        self.assert_metadata("unknown key", None, "test_with_metadata")

    def assert_metadata(self, name, expected_value, test_name):
        self.assert_equal(expected_value,
                          self.TestCase(test_name).get_metadata(name))



import time
import pikzie

class TestCompatibility(pikzie.TestCase):
    """Tests for version compatibility"""

    def test_strptime(self):
        self.assert_equal((2008, 11, 10, 0, 0, 0, 0, 315, -1),
                          time.strptime('2008 Nov 10 00:00:00',
                                        '%Y %b %d %H:%M:%S'))



# Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import types
import difflib
import pprint

def format(object):
    return pprint.pformat(object)

_re_class = type(re.compile(""))
def _re_flags(pattern):
    result = ""
    if isinstance(pattern, _re_class):
        if pattern.flags & re.IGNORECASE: result += "i"
        if pattern.flags & re.LOCALE: result += "l"
        if pattern.flags & re.MULTILINE: result += "m"
        if pattern.flags & re.DOTALL: result += "s"
        if pattern.flags & re.UNICODE: result += "u"
        if pattern.flags & re.VERBOSE: result += "x"
    return result

def _re_flags_repr(pattern):
    flags = []
    if isinstance(pattern, _re_class):
        if pattern.flags & re.IGNORECASE: flags.append("re.IGNORECASE")
        if pattern.flags & re.LOCALE: flags.append("re.LOCALE")
        if pattern.flags & re.MULTILINE: flags.append("re.MULTILINE")
        if pattern.flags & re.DOTALL: flags.append("re.DOTALL")
        if pattern.flags & re.UNICODE: flags.append("re.UNICODE")
        if pattern.flags & re.VERBOSE: flags.append("re.VERBOSE")
    if len(flags) == 0:
        return None
    else:
        return " | ".join(flags)

def format_re(pattern):
    re_flags = _re_flags(pattern)
    if hasattr(pattern, "pattern"):
        pattern = pattern.pattern
    pattern = format(pattern)
    return "/%s/%s" % (pattern[1:-1], re_flags)

def format_re_repr(pattern):
    re_flags_repr = _re_flags_repr(pattern)
    if hasattr(pattern, "pattern"):
        pattern = pattern.pattern
    pattern = pprint.pformat(pattern)
    if re_flags_repr:
        return "re.compile(%s, %s)" % (pattern, re_flags_repr)
    else:
        return pattern

def format_callable_object(callable_object):
    if hasattr(callable_object, "im_class"):
        cls = callable_object.im_class
        return "%s.%s.%s" % (cls.__module__,
                             cls.__class__.__name__,
                             callable_object.__name__)
    else:
        return "%s.%s" % (callable_object.__module__,
                          callable_object.__name__)

def format_call_arguments(args, kw_args):
    formatted_args = [format(arg) for arg in args]
    formatted_args.extend(["%s=%s" % (format(key), format(value))
                           for (key, value) in kw_args.items()])
    return ", ".join(formatted_args)

def format_call(callable_object, args, kw_args):
    return "%s(%s)" % (format_callable_object(callable_object),
                       format_call_arguments(args, kw_args))

def format_exception_class(exception_class):
    if issubclass(exception_class, Exception) or \
            issubclass(exception_class, types.ClassType):
        return str(exception_class)
    else:
        return format(exception_class)

def format_for_diff(object):
    if not isinstance(object, str):
        object = format(object)
    return object

def format_diff(string1, string2):
    def ensure_newline(string):
        if string.endswith("\n"):
            return string
        else:
            return string + "\n"
    diff = difflib.ndiff(ensure_newline(string1).splitlines(True),
                         ensure_newline(string2).splitlines(True))
    return "".join(diff).rstrip()

def is_interested_diff(diff):
    if not diff:
        return False
    if not re.search("^[-+]", diff, re.MULTILINE):
        return False
    if re.search("^[ ?]", diff, re.MULTILINE):
        return True
    if re.search("(?:.*\n){2,}", diff):
        return True
    return False

def need_fold(diff):
    return re.search("^[-?+].{79}", diff, re.MULTILINE)

def fold(string):
    def fold_line(line):
        return re.subn("(.{78})", "\\1\n", line)[0]
    return "\n".join([fold_line(line) for line in string.split("\n")])

def format_folded_diff(string1, string2):
    return format_diff(fold(string1), fold(string2))

def append_diff(message, target1, target2):
    formatted_target1 = format_for_diff(target1)
    formatted_target2 = format_for_diff(target2)
    diff = format_diff(formatted_target1, formatted_target2)
    if is_interested_diff(diff):
        message = "%s\n\ndiff:\n%s" % (message, diff)
    if need_fold(diff):
        folded_diff = format_folded_diff(formatted_target1, formatted_target2)
        message = "%s\n\nfolded diff:\n%s" % (message, folded_diff)
    return message




import os
import pikzie

class TestLoader(pikzie.TestCase):
    """Tests for TestLoader"""

    def setup(self):
        self.fixture_dir = os.path.join("test", "fixtures", "tests")
        self.loader = pikzie.TestLoader(base_dir=self.fixture_dir,
                                        priority_mode=False)

    def test_find_targets(self):
        def build_full_path(name):
            return "test_%s.py" % name
        base_dir, targets = self.loader._find_targets()
        self.assert_equal((os.path.abspath(self.fixture_dir),
                           sorted(map(build_full_path,
                                      ["module_base", "xxx", "yyy", "zzz"]))),
                          (base_dir, sorted(targets)))

    def test_load_modules(self):
        modules = self.loader._load_modules()
        module_names = map(lambda module: module.__name__, modules)
        self.assert_equal(["test_module_base",
                           "test_xxx",
                           "test_yyy",
                           "test_zzz"],
                          sorted(module_names))

    def test_collect_test_cases(self):
        test_cases = self.loader.collect_test_cases()
        test_case_names = map(lambda case: case.__name__, test_cases)
        self.assert_equal(["TestXXX1",
                           "TestXXX2",
                           "TestYYY",
                           'test_module_base',
                           'test_xxx',
                           'test_yyy'],
                          sorted(test_case_names))

    def test_collect_test_cases_with_test_case_name_filter(self):
        self.loader.test_case_names = ["TestXXX1"]
        test_cases = self.loader.collect_test_cases()
        test_case_names = map(lambda case: case.__name__, test_cases)
        self.assert_equal(["TestXXX1"], sorted(test_case_names))

    def test_collect_test_cases_with_test_case_name_regexp_filter(self):
        self.loader.test_case_names = ["/XXX/"]
        test_cases = self.loader.collect_test_cases()
        test_case_names = map(lambda case: case.__name__, test_cases)
        self.assert_equal(["TestXXX1", "TestXXX2"], sorted(test_case_names))

    def test_create_test_suite(self):
        test_suite = self.loader.create_test_suite()
        self.assert_equal(
            ["test_module_base.test_top_level1",
             "test_module_base.test_top_level2",
             "test_xxx.TestXXX1.test_one",
             "test_xxx.TestXXX1.test_two",
             "test_yyy.TestYYY.test_xyz"],
            sorted(self._collect_test_names(test_suite)))

    def test_create_test_suite_with_test_name_filter(self):
        self.loader.test_names = ["test_one", "test_xyz"]
        test_suite = self.loader.create_test_suite()
        self.assert_equal(["test_xxx.TestXXX1.test_one",
                           "test_yyy.TestYYY.test_xyz"],
                          sorted(self._collect_test_names(test_suite)))

    def test_create_test_suite_with_test_name_regexp_filter(self):
        self.loader.test_names = ["/one|xyz/"]
        test_suite = self.loader.create_test_suite()
        self.assert_equal(["test_xxx.TestXXX1.test_one",
                           "test_yyy.TestYYY.test_xyz"],
                          sorted(self._collect_test_names(test_suite)))

    def test_create_test_suite_with_filters(self):
        self.loader.test_case_names = "TestXXX1"
        self.loader.test_names = ["test_one", "test_two"]
        test_suite = self.loader.create_test_suite()
        self.assert_equal(["test_xxx.TestXXX1.test_one",
                           "test_xxx.TestXXX1.test_two"],
                          sorted(self._collect_test_names(test_suite)))

    def test_create_test_suite_with_regexp_filters(self):
        self.loader.test_case_names = ["/XXX/", "/YYY/"]
        self.loader.test_names = ["/one|xyz/"]
        test_suite = self.loader.create_test_suite()
        self.assert_equal(["test_xxx.TestXXX1.test_one",
                           "test_yyy.TestYYY.test_xyz"],
                          sorted(self._collect_test_names(test_suite)))

    def _collect_test_names(self, test_suite):
        names = []
        for test_case_runner in test_suite._tests:
            names.extend([test.id() for test in test_case_runner.tests()])
        return names
